package net.user1.reactor.motion {
  import flash.display.DisplayObject;
  import flash.events.TimerEvent;
  import flash.geom.Point;
  import flash.utils.Timer;
  
  import net.user1.reactor.Reactor;
  import net.user1.reactor.ReactorEvent;
  
  /**
   * Broadcasts the current client's mouse positions to other connected clients.
   * Positions are sampled relative to the object specified via the
   * PointerPositionBroadcaster's constructor's positionTarget parameter.
   * The PointerPositionBroadcaster uses a PositionBroadcaster to send
   * positions.
   * 
   * @since ReactorGUI 1.0.0
   * 
   * @see PositionBroadcaster 
   */  
  public class PointerPositionBroadcaster {
// =============================================================================    
//  VARIABLES
// =============================================================================
    public static var DEFAULT_SAMPLE_RATE:int = 50;
    
    protected var reactor:Reactor;
    protected var positionTarget:DisplayObject;
    protected var sampleTimer:Timer;
    protected var lastMouseX:int;
    protected var lastMouseY:int;
    protected var positions:PositionList;
    protected var positionBroadcaster:PositionBroadcaster;
    
// =============================================================================    
//  CONSTRUCTOR
// =============================================================================    
    /**
     * Constructor
     *  
     * @param reactor The Reactor object through which position list updates
     *                will be sent.
     * @param positionTarget The display object relative to which mouse positions
     *                will be sampled.
     * @param sampleRate The rate at which to collect mouse position samples.
     * @param sendRate The rate at which to send position list updates.
     * @param scope   The attribute scope for position list updates. To limit
     *                position-list recipients to a single room, set
     *                <code>scope</code> to that room (e.g., 
     *                "examples.avatarchat"). 
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function PointerPositionBroadcaster(reactor:Reactor,
                                               positionTarget:DisplayObject,
                                               sampleRate:int = 50,
                                               sendRate:int = 300,
                                               scope:String = null) {
      this.positionTarget = positionTarget;
      this.reactor = reactor;
      
      reactor.addEventListener(ReactorEvent.READY, readyListener);
      reactor.addEventListener(ReactorEvent.CLOSE, closeListener);
      
      positions  = new PositionList();
      positionBroadcaster = new PositionBroadcaster(reactor, sendRate, scope);
      positionBroadcaster.setPositionList(positions);
      
      sampleTimer = new Timer(sampleRate, 0);
      sampleTimer.addEventListener(TimerEvent.TIMER, sampleTimerListener);
      if (reactor.isReady()) {
        start();
      }
    }

// =============================================================================    
//  ACTIVATION/DEACTIVATION
// =============================================================================    
    /**
     * Enables position-list broadcasts. Note that if Reactor is not
     * currently connected to Union, no updates are sent, even if the 
     * PositionBroadcaster is enabled.
     * 
     * @since ReactorGUI 1.0.0
     */      
    public function start ():void {
      sampleTimer.start();
    }
    
    /**
     * Disables position-list broadcasts.
     * 
     * 
     * @since ReactorGUI 1.0.0
     */
    public function stop ():void {
      sampleTimer.stop();
    }
    
// =============================================================================    
//  POSITION LIST ATTRIBUTE MANAGEMENT
// =============================================================================   
    /**
     * Specifies the attribute name used by this PointerPositionBroadcaster to send
     * position list updates to other clients.
     * 
     * @since ReactorGUI 1.0.0
     */     
    public function setPositionListAttributeName (name:String):void {
      positionBroadcaster.setPositionListAttributeName(name);
    }
    
    /**
     * Returns the attribute name used by this PointerPositionBroadcaster to send
     * position list updates to other clients.
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function getPositionListAttributeName ():String {
      return positionBroadcaster.getPositionListAttributeName();
    }

// =============================================================================    
//  MOUSE-POSITION SAMPLE TIMER LISTENER
// =============================================================================        
    protected function sampleTimerListener (e:TimerEvent):void {
      var x:int;
      var y:int;
      
      x = positionTarget.mouseX;
      y = positionTarget.mouseY;
        
      if (lastMouseX != x || lastMouseY != y) {
        positions.addPosition(new Point(x, y));
        lastMouseX = x;
        lastMouseY = y;
      }
    }
    
// =============================================================================    
//  REACTOR LISTENERS
// =============================================================================        
    protected function readyListener (e:ReactorEvent):void {
      start();
    }
    
    protected function closeListener (e:ReactorEvent):void {
      stop();
    }
    
// =============================================================================    
//  DISPOSAL
// =============================================================================    
    /**
     * Disposes of the resources used by this object, preparing it for garbage
     * collection.
     * 
     * @since ReactorGUI 1.0.0
     */     
    public function dispose ():void {
      sampleTimer.stop();
      sampleTimer.removeEventListener(TimerEvent.TIMER, sampleTimerListener);
      sampleTimer = null;
      
      reactor.removeEventListener(ReactorEvent.READY, readyListener);
      reactor.removeEventListener(ReactorEvent.CLOSE, closeListener);
      
      positionBroadcaster = null;
      positions = null;
      reactor = null;
      positionTarget = null;
    }
  }
}