package net.user1.reactor.motion {
  import flash.events.TimerEvent;
  import flash.utils.Timer;
  
  import net.user1.reactor.Reactor;
  import net.user1.reactor.ReactorEvent;
  
  /**
   * <p>
   * Broadcasts the positions in a PositionList to interested clients.
   * The PositionBroadcaster broadcasts updates to other clients by setting
   * an attribute on itself at a specified send-rate. By default, the
   * attribute is global, so all clients that have specified interest in the
   * current client (for example, by joining a room containing the current
   * client) receive position-list attribute updates. To limit the updates to
   * a single room, set the PositionBroadcaster's constructor's <code>scope</code>
   * parameter to that room's fully qualified identifier 
   * (e.g., "examples.avatarchat").
   * </p>
   * 
   * <p>
   * Every time the PositionBroadcaster sends a list of positions from its
   * corresponding PositionList, it clears the contents of that list. If new
   * positions are subsequently added to the PositionList, they are sent during
   * the next broadcast.
   * </p>
   * 
   * @since ReactorGUI 1.0.0
   */
  public class PositionBroadcaster {
// =============================================================================    
//  VARIABLES
// =============================================================================    
    /**
     * The default attribute name used for PositionList updates. To specify
     * a custom name, use setPositionListAttributeName().
     * 
     * @since ReactorGUI 1.0.0
     * 
     * @see #setPositionListAttributeName() 
     */    
    public static const DEFAULT_POSITION_LIST_ATTRIBUTE_NAME:String = "POS_LIST";
    
    /**
     * The default rate at which to send PositionList updates.
     * 
     * @since ReactorGUI 1.0.0
     * 
     * @see #setPositionListAttributeName() 
     */    
    public static const DEFAULT_SEND_RATE:int = 300;
    protected var positionListAttributeName:String; 
    protected var reactor:Reactor;
    protected var sendTimer:Timer;
    protected var scope:String;
    protected var positions:PositionList;
    
// =============================================================================    
//  CONSTRUCTOR
// =============================================================================  
    /**
     * Constructor
     *  
     * @param reactor The Reactor object through which position list updates
     *                will be sent.
     * @param sendRate The rate at which to send position list updates.
     * @param scope   The attribute scope for position list updates. To limit
     *                position-list recipients to a single room, set
     *                <code>scope</code> to that room (e.g., 
     *                "examples.avatarchat"). 
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function PositionBroadcaster (reactor:Reactor,
                                         sendRate:int = PositionBroadcaster.DEFAULT_SEND_RATE,
                                         scope:String = null) {
      this.reactor = reactor;
      this.scope = scope;
      
      positionListAttributeName = PositionBroadcaster.DEFAULT_POSITION_LIST_ATTRIBUTE_NAME;
      
      reactor.addEventListener(ReactorEvent.READY, readyListener);
      reactor.addEventListener(ReactorEvent.CLOSE, closeListener);
      
      sendTimer = new Timer(sendRate, 0);
      sendTimer.addEventListener(TimerEvent.TIMER, sendTimerListener);
      if (reactor.isReady()) {
        start();
      }
    }
    
// =============================================================================    
//  DEPENDENCIES
// =============================================================================    
    /**
     * Specifies the PositionList object whose positions will be broadcast to
     * other clients by this PositionBroadcaster.
     *  
     * @param positionList A PositionList object.
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function setPositionList (positionList:PositionList):void {
      positions = positionList;
    }
    
    /**
     * Returns the PositionList object whose positions are broadcast to
     * other clients by this PositionBroadcaster.
     *  
     * @return A PositionList object.
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function getPositionList ():PositionList {
      return positions;
    }
    
// =============================================================================    
//  POSITION LIST ATTRIBUTE MANAGEMENT
// =============================================================================   
    /**
     * Specifies the attribute name used by this PositionBroadcaster to send
     * position list updates to other clients.
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function setPositionListAttributeName (name:String):void {
      positionListAttributeName = name;
    }
    
    /**
     * Returns the attribute name used by this PositionBroadcaster to send
     * position list updates to other clients.
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function getPositionListAttributeName ():String {
      return positionListAttributeName;
    }

// =============================================================================    
//  ACTIVATION/DEACTIVATION
// =============================================================================    
    /**
     * Enables position-list broadcasts. Note that if Reactor is not
     * currently connected to Union, no updates are sent, even if the 
     * PositionBroadcaster is enabled.
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function start ():void {
      sendTimer.start();
    }
    
    /**
     * Enables position-list broadcasts.
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function stop ():void {
      sendTimer.stop();
    }
    
// =============================================================================    
//  REACTOR LISTENERS
// =============================================================================    
    protected function readyListener (e:ReactorEvent):void {
      start();
    }
    
    protected function closeListener (e:ReactorEvent):void {
      stop();
    }
    
// =============================================================================    
//  SEND-TIMER LISTENER
// =============================================================================  
    /**
     * Triggered when the "send timer" ticks, causing this client to broadcast
     * a position list.
     */
    protected function sendTimerListener (e:TimerEvent):void {
      if (reactor.isReady()) {
        if (positions.length > 0) {
          reactor.self().setAttribute(
            positionListAttributeName,
            positions.serialize(),
            scope);
        }
        positions.clear();
      }
    }
    
// =============================================================================    
//  DISPOSAL
// =============================================================================    
    /**
     * Disposes of the resources used by this object, preparing it for garbage
     * collection.
     * 
     * @since ReactorGUI 1.0.0
     */ 
    public function dispose ():void {
      sendTimer.stop();
      sendTimer.removeEventListener(TimerEvent.TIMER, sendTimerListener);
      
      reactor.removeEventListener(ReactorEvent.READY, readyListener);
      reactor.removeEventListener(ReactorEvent.CLOSE, closeListener);
      
      sendTimer = null;
      positions = null;
      reactor = null;
    }
  }
}