package net.user1.reactor.ui {
  import flash.events.MouseEvent;
  
  import mx.containers.HBox;
  import mx.controls.LinkButton;
  import mx.core.UIComponent;
  import mx.managers.PopUpManager;
  
  import net.user1.reactor.AccountEvent;
  import net.user1.reactor.AccountManager;
  import net.user1.reactor.ConnectionState;
  import net.user1.reactor.IClient;
  import net.user1.utils.EventUtil;
  
  /**
   * A three-button panel for registering, logging in, and logging off a user
   * account. Registration launches a RegisterWindow instance. Login launches
   * a LoginWindow instance. Buttons are automatically disabled and enabled
   * based on the current client's connection state. For example, when the 
   * current client is already logged in, the "Login" button is disabled.
   *
   * @since ReactorGUI 1.0.0
   * 
   * @see LoginWindow
   * @see RegisterWindow
   */
  public class AccountBar extends HBox {
// =============================================================================    
//  VARIABLES
// =============================================================================    
    private var _accountManager:AccountManager;

    private var registerButton:LinkButton;
    private var loginButton:LinkButton;
    private var logoffButton:LinkButton;
    
// =============================================================================    
//  CONSTRUCTOR
// =============================================================================    
    public function AccountBar () {
      buildUI();
      refreshButtons();
    }
    
// =============================================================================    
//  UI CREATION
// =============================================================================    
    
    protected function buildUI ():void {
      registerButton = new LinkButton();
      registerButton.label = "Register";
      registerButton.addEventListener(MouseEvent.CLICK, registerClickListener);
      
      loginButton = new LinkButton();
      loginButton.label = "Login";
      loginButton.addEventListener(MouseEvent.CLICK, loginClickListener);
      
      logoffButton = new LinkButton();
      logoffButton.label = "Logoff";
      logoffButton.addEventListener(MouseEvent.CLICK, logoffClickListener);
      
      addChild(loginButton);
      addChild(logoffButton);
      addChild(registerButton);
    }
    
// =============================================================================    
//  DEPENDENCIES   
// =============================================================================    
    
    /**
     * @private
     */
    public function set accountManager (value:AccountManager):void {
      _accountManager = value;
      EventUtil.migrateListeners(_accountManager, value,
                                 [AccountEvent.LOGIN, loginListener,
                                  AccountEvent.LOGOFF, logoffListener]);
      refreshButtons();
    }
    
    /**
     * A Reactor AccountManager reference that is 
     * used to perform account registration and login.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get accountManager ():AccountManager {
      return _accountManager;
    }
    
// =============================================================================
// CURRENT-CLIENT EVENT LISTENERS
// =============================================================================
    
    /**
     * Triggered when a user logs in.
     */
    protected function loginListener (e:AccountEvent):void {
      if (e.getAccount().isSelf()) {
        refreshButtons();
      }
    }
    
    /**
     * Triggered when a user logs off.
     */
    protected function logoffListener (e:AccountEvent):void {
      if (e.getAccount().isSelf()) {
        refreshButtons();
      }
    }
    
// =============================================================================
// UI EVENT LISTENERS
// =============================================================================
    
    /**
     * Listener triggered when the "Register" button is clicked.
     */
    protected function registerClickListener (e:MouseEvent):void {
      var popup:RegisterWindow = RegisterWindow(
        PopUpManager.createPopUp(UIComponent(this.parentApplication), RegisterWindow, true));
      popup.accountManager = accountManager;
      PopUpManager.centerPopUp(popup);
    }
    
    /**
     * Listener triggered when the "Login" button is clicked.
     */
    protected function loginClickListener (e:MouseEvent):void {
      var popup:LoginWindow = LoginWindow(
        PopUpManager.createPopUp(UIComponent(this.parentApplication), LoginWindow, true));
      popup.accountManager = accountManager;
      PopUpManager.centerPopUp(popup);
    }
    
    /**
     * Listener triggered when the "Logoff" button is clicked.
     */
    protected function logoffClickListener (e:MouseEvent):void {
      if (accountManager.selfAccount().getConnectionState() == ConnectionState.LOGGED_IN) {
        accountManager.logoff();
      }
    }

// =============================================================================
// UI MANAGEMENT
// =============================================================================
    
    protected function refreshButtons ():void {
      if (accountManager == null) {
        registerButton.enabled = false;
        registerButton.mouseEnabled = false;
        loginButton.enabled = false;
        loginButton.mouseEnabled = false;
        logoffButton.enabled = false;
        logoffButton.mouseEnabled = false;
        return;
      } 
      
      registerButton.enabled = true;
      registerButton.mouseEnabled = true;
      
      if (accountManager.selfAccount()
          && accountManager.selfAccount().isLoggedIn()) {
        loginButton.enabled = false;
        loginButton.mouseEnabled = false;
        logoffButton.enabled = true;
        logoffButton.mouseEnabled = true;
      } else {
        loginButton.enabled = true;
        loginButton.mouseEnabled = true;
        logoffButton.enabled = false;
        logoffButton.mouseEnabled = false;
      }
    }
  }
}