package net.user1.reactor.ui {
  import flash.events.TimerEvent;
  import flash.utils.Timer;
  
  import mx.collections.ArrayCollection;
  import mx.collections.Sort;
  import mx.collections.SortField;
  import mx.controls.DataGrid;
  import mx.controls.dataGridClasses.DataGridColumn;
  import mx.controls.dataGridClasses.DataGridItemRenderer;
  import mx.events.FlexEvent;
  
  import net.user1.reactor.AccountManager;
  import net.user1.reactor.AccountManagerEvent;
  import net.user1.reactor.UserAccount;
  import net.user1.utils.EventUtil;
  
  /**
   * Displays a list of user accounts currently known to Reactor. To display all
   * accounts on the server, first invoke AccountManager's watchForUserAccounts()
   * method.
   * 
   * @since ReactorGUI 1.0.0
   */
  public class AccountGrid extends DataGrid {
    protected var _accountManager:AccountManager;
    private var refreshTimer:Timer;
    
    protected const stateDescriptions:Array = ["Unknown",
                                               "Not connected",
                                               "Ready",
                                               "Connection in progress",
                                               "Disconnection in progress",
                                               "Logged in",
                                               "Deleted"];

    public function AccountGrid () {
      dataProvider = new ArrayCollection();
      var accountColumn:DataGridColumn = new DataGridColumn();
      accountColumn.dataField = "userID";
      accountColumn.headerText = "User ID";
      var stateColumn:DataGridColumn = new DataGridColumn();
      stateColumn.dataField = "state";
      stateColumn.headerText = "State";
      stateColumn.sortable = false;
      
      columns = [accountColumn, stateColumn];
      
      refreshTimer = new Timer(5000);
      refreshTimer.addEventListener(TimerEvent.TIMER, refreshTimerListener);
      refreshTimer.start();
      
      var sort:Sort = new Sort();
      sort.fields = [new SortField("userID", true, false)];
      dataProvider.sort = sort;
    }
    
// =============================================================================
// REFRESH TIMER
// =============================================================================

    protected function refreshTimerListener (e:TimerEvent):void {
      refreshAccountDetails();
    }
    
    public function get accountManager():AccountManager {
      return _accountManager;
    }

    public function set accountManager(value:AccountManager):void {
      EventUtil.migrateListeners(_accountManager, value, 
                                 [AccountManagerEvent.ACCOUNT_ADDED, addAccountListener,
                                  AccountManagerEvent.ACCOUNT_REMOVED, removeAccountListener,
                                  AccountManagerEvent.SYNCHRONIZE, synchronizeListener]);
      _accountManager = value;
      clearProvider();
      populateProvider();
    }
    
    protected function populateProvider ():void {
      if (_accountManager != null) {
        for each (var account:UserAccount in _accountManager.getAccounts()) {
          dataProvider.source.push({userID:account.getUserID(), 
                                    state:stateDescriptions[account.getConnectionState()+1]});
        }
        dataProvider.refresh();
      }
    }

    protected function addAccountListener (e:AccountManagerEvent):void {
      addItemToGrid(e.getUserID(), 
                    stateDescriptions[_accountManager.getAccount(e.getUserID()).getConnectionState()+1]);
    }
    
    protected function removeAccountListener (e:AccountManagerEvent):void {
      removeItemFromGrid(e.getUserID());
    }
    
    protected function synchronizeListener (e:AccountManagerEvent):void {
      dataProvider.refresh();
    }
    
    protected function addItemToGrid (userID:String, state:String):void {
      dataProvider.source.push({userID:userID, state:state});
    }    
    
    protected function removeItemFromGrid (userID:String):void {
      var item:Object;
      var length:int = dataProvider.source.length;
      var source:Array = dataProvider.source; 
      for (var i:int = length; --i >= 0;) {
        item = source[i];
        if (userID == item.userID) {
          source.splice(i, 1);
          return;
        }
      }
    }
    
    protected function refreshAccountDetails ():void {
      var item:Object;
      
      for each (var renderer:DataGridItemRenderer in visibleData) {
        item       = renderer.data;
        item.state = stateDescriptions[_accountManager.getAccount(item.userID).getConnectionState()+1];
      }

      // Record grid state before refreshing provider
      var vPos:int = verticalScrollPosition;
      var oldSelectedIndex:int = selectedIndex;
      
      // Refresh provider
      dataProvider.refresh();
      
      // If selected item is now gone, the selection will have changed, in which
      // case, manually dispatch VALUE_COMMIT because the item was removed without
      // the provider's knowledge.
      if (oldSelectedIndex != selectedIndex) {
        dispatchEvent(new FlexEvent(FlexEvent.VALUE_COMMIT));
      }
      
      // Restore scroll position
      verticalScrollPosition = vPos;
    }
    
    protected function clearProvider ():void {
      dataProvider.source = new Array();
    }
  }
}