package net.user1.reactor.ui {
  import flash.events.Event;
  
  import mx.controls.TextArea;
  import mx.events.ScrollEvent;
  import mx.events.ScrollEventDetail;
  import mx.events.ScrollEventDirection;

  /**
   * A TextArea that automatically scrolls to the bottom when new text is 
   * added, as is required in chat and logging interfaces. When the user scrolls
   * the text field up, the automatic-scroll behaviour is temporarily disabled. 
   * Based on Flex's TextArea class.
   * 
   * @since ReactorGUI 1.0.0
   */
  public class AutoScrollTextArea extends TextArea {
// =============================================================================    
//  VARIABLES
// =============================================================================    
    
    private var _maxLength:int;
    private var _autoScrollEnabled:Boolean;
    
// =============================================================================    
//  CONSTRUCTOR
// =============================================================================    
    /**
     * Constructor
     */
    public function AutoScrollTextArea ():void {
      _autoScrollEnabled = true;
      addEventListener(Event.ENTER_FRAME, enterFrameListener);
      addEventListener(ScrollEvent.SCROLL, scrollListener);
    }
    
// =============================================================================    
//  MAX LENGTH
// =============================================================================    
    /**
     * The maximum allowed number of characters for the text field.
     * When the text field's content exceeds the specified maxLength, 
     * characters at the top of the text field (starting at index 0) are 
     * automatically removed.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get maxLength ():int {
      return _maxLength;
    }

    /**
     * @private
     */
    public function set maxLength (value:int):void {
      _maxLength = value;
    }
    
// =============================================================================    
//  TEXT CONTENT
// =============================================================================    

    /**
     * The text field's content. The AutoScrollTextArea class overrides
     * Flex's TextArea's text setter in order to truncate text if necessary at
     * assignment time.
     * 
     * @since ReactorGUI 1.0.0
     */
    override public function set text (value:String):void {
      // Remember scroll position and selection position
      var prevVerticalScrollPosition:int = verticalScrollPosition;
      var prevHorizontalScrollPosition:int = horizontalScrollPosition;
      var prevSelectionBeginIndex:int = textField.selectionBeginIndex;
      var prevSelectionEndIndex:int = textField.selectionEndIndex;
      
      // Add the new text
      super.text = value;
        
      // Truncate the text if necessary
      var numTrimmedChars:int = truncate();
      
      // Restore the scroll position
      verticalScrollPosition = prevVerticalScrollPosition;
      horizontalScrollPosition = prevHorizontalScrollPosition;
      
      // Restore the selection
      textField.setSelection(Math.max(0, prevSelectionBeginIndex - numTrimmedChars), 
                             Math.max(0, prevSelectionEndIndex - numTrimmedChars)); 
      
      // Force a redraw
      invalidateDisplayList();
    }

    /**
     * Adds new text to the end of the text field, truncating characters if 
     * necessary.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function appendText (value:String):void {
      // Remember scroll position and selection position
      var prevVerticalScrollPosition:int = verticalScrollPosition;
      var prevHorizontalScrollPosition:int = horizontalScrollPosition;
      var prevSelectionBeginIndex:int = textField.selectionBeginIndex;
      var prevSelectionEndIndex:int = textField.selectionEndIndex;
      
      // Add the new text
      textField.appendText(value)
        
      // Truncate the text if necessary
      var numTrimmedChars:int = truncate();
      
      // Restore the scroll position
      verticalScrollPosition = prevVerticalScrollPosition;
      horizontalScrollPosition = prevHorizontalScrollPosition;
      
      // Restore the selection
      textField.setSelection(Math.max(0, prevSelectionBeginIndex - numTrimmedChars),
                             Math.max(0, prevSelectionEndIndex - numTrimmedChars)); 
      
      // Force a redraw
      invalidateDisplayList();
    }
    
// =============================================================================    
//  TEXT TRUNCATION
// =============================================================================    
    
    protected function truncate ():int {
      // If the max characters has been exceeded...
      if (_maxLength > 0
          && textField.length > _maxLength) {
        // Get the index of the first newline character in the text that will 
        // remain after truncation. Mac uses CR, Unix LF, Win CRLF
        var crIndex:int = textField.text.indexOf("\r", textField.length - _maxLength);
        var lfIndex:int = textField.text.indexOf("\n", textField.length - _maxLength);
        var newlineIndex:int = (lfIndex == -1) ? crIndex : lfIndex;
        
        // Trim to first newline, or to maxLength if no newline was found
        var trimTo:int;
        if (newlineIndex != -1) {
          trimTo = newlineIndex+1;
          textField.replaceText(0, trimTo, "");
        } else {
          trimTo = textField.length - _maxLength;
          textField.replaceText(0, trimTo, "");
        }
        return trimTo;
      }
      
      invalidateDisplayList();
      return 0;
    }
    
// =============================================================================    
//  SCROLLING
// =============================================================================    
    
    protected function scrollListener (e:ScrollEvent):void {
      // Disable automatic scrolling if the user moves the vertical scrollbar up
      if (e.detail == ScrollEventDetail.THUMB_TRACK) {
        if (e.direction == ScrollEventDirection.VERTICAL) {
          if (e.position < maxVerticalScrollPosition) {
            _autoScrollEnabled = false;
          } else {
            _autoScrollEnabled = true;
          }
        }
      }
    }
    
    protected function enterFrameListener (e:Event):void {
      if (!_autoScrollEnabled) {
        return;
      }
      // Brute-force: send the scrollbar to one-past-the-bottom to workaround
      // an apparent scrollbar positioning bug in Flex's TextArea component
      verticalScrollPosition = maxVerticalScrollPosition + 1;
      invalidateDisplayList();
    }
    
// =============================================================================    
//  DISPOSAL
// =============================================================================    
    
    /**
     * Disposes of the resources used by this object, preparing it for garbage
     * collection.
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function dispose ():void {
      removeEventListener(Event.ENTER_FRAME, enterFrameListener);
    }
  }
}