package net.user1.reactor.ui {
  import com.gskinner.motion.GTween;
  
  import flash.display.DisplayObject;
  import flash.display.Sprite;
  import flash.geom.Point;
  
  import net.user1.reactor.AttributeEvent;
  import net.user1.reactor.ClientEvent;
  import net.user1.reactor.IClient;
  import net.user1.reactor.motion.PositionBroadcaster;
  import net.user1.reactor.motion.PositionList;
  
  /**
   * <p>
   * A visual representation of a user on the screen. Each Avatar instance's
   * position is determined by the value of a position list attribute
   * specified setPositionListAttributeName() and setPositionListAttributeScope().
   * If the value of that attribute changes, the Avatar animates through the
   * list of new positions. Use the PositionList class to generate a list of
   * positions for use with the Avatar class's position list attribute.
   * </p>
   * 
   * <p>
   * Note that the Avatar class extends Sprite, and, therefore, can be used in
   * non-Flex applications.
   * </p>
   * 
   * @since ReactorGUI 1.0.0
   */
  public class Avatar extends Sprite {
// =============================================================================    
//  VARIABLES
// =============================================================================    
    protected var positions:PositionList;
    protected var icon:DisplayObject;
    protected var fader:GTween;
    protected var animator:GTween;
    protected var client:IClient;
    protected var positionListAttributeScope:String;
    protected var positionListAttributeName:String;
    
// =============================================================================    
//  CONSTRUCTOR
// =============================================================================    
    /**
     * Constructor
     */
    public function Avatar (speed:int = 100) {
      positions = new PositionList();
      fader     = new GTween(this);
      setFadeInDuration();
      
      animator  = new GTween(this);
      animator.duration = speed/1000;
      animator.onComplete = animationEndListener;
      setPositionListAttributeName();
    }
    
// =============================================================================    
//  VISUAL SETTINGS
// =============================================================================    
    
    /**
     * Specifies the DisplayObject instance that will be used for this
     * Avatar's icon.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function setIcon (icon:DisplayObject):void {
      if (this.icon != null) {
        removeChild(this.icon);
      }
      this.icon = icon;
      addChild(this.icon);
    }
    
    /**
     * Returns the DisplayObject instance for this
     * Avatar's icon.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function getIcon ():DisplayObject {
      return icon;
    }
    
    /**
     * Specifies the duration of the fade-in animation that plays when this
     * Avatar is added to the screen for the first time.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function setFadeInDuration (milliseconds:int = 500):void {
      fader.duration = milliseconds/1000;
    }
    
    /**
     * Specifies the length of time, in milliseconds, required for this Avatar
     * to travel from its current position to its next position.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function setSpeed (speed:uint):void {
      animator.duration = speed;
    }
    
// =============================================================================    
//  POSITION LIST ATTRIBUTE
// =============================================================================    
    
    /**
     * Specifies the scope of the attribute to use for this Avatar's position
     * list. The attribute must be set by the underlying IClient object.
     * See setClient().
     * 
     * @since ReactorGUI 1.0.0
     * 
     * @see #setClient()
     */
    public function setPositionListAttributeScope (scope:String = null):void {
      positionListAttributeScope = scope;
    }
    
    /**
     * Specifies the name of the attribute to use for this Avatar's position
     * list. The attribute must be set by the underlying IClient object.
     * See setClient().
     * 
     * @since ReactorGUI 1.0.0
     * 
     * @see #setClient()
     */
    public function setPositionListAttributeName (name:String = PositionBroadcaster.DEFAULT_POSITION_LIST_ATTRIBUTE_NAME):void {
      positionListAttributeName = name;
    }
    
// =============================================================================    
//  DEPENDENCIES
// =============================================================================    
    
    /**
     * Returns the underlying IClient object that provides data for this 
     * Avatar. 
     * 
     * @since ReactorGUI 1.0.0
     */
    public function getClient ():IClient {
      return client;
    }
    
    /**
     * Specifies the underlying IClient object that will provide data
     * for this Avatar. The IClient object must define a shared attribute whose 
     * value contains the specified client's position list. By default, the
     * attribute must be a global attribute whose name matches the value of
     * PositionBroadcaster.DEFAULT_POSITION_LIST_ATTRIBUTE_NAME. However, a
     * custom attribute name and scope can be specified via 
     * setPositionListAttributeName() and setPositionListAttributeScope().
     * 
     * @since ReactorGUI 1.0.0
     * 
     * @see setPositionListAttributeScope()
     * @see setPositionListAttributeName()
     */
    public function setClient (client:IClient):void {
      var oldClient:IClient = this.client;
      this.client = client;
      if (oldClient == client) {
        return;
      }
      if (oldClient != null) {
        oldClient.removeEventListener(AttributeEvent.UPDATE, 
          updateAttributeListener);
      }
      if (client != null) {
        client.addEventListener(AttributeEvent.UPDATE, 
          updateAttributeListener);
        var posList:PositionList = new PositionList();
        posList.deserialize(client.getAttribute(positionListAttributeName, 
                                                positionListAttributeScope));
        setPosition(posList.getPositionAt(0).x, posList.getPositionAt(0).y);
        posList.shift();
        
        if (fader.duration > 0) {
          this.alpha = 0;
          fader.setValue("alpha", 1);
        }
      }
    }
    
// =============================================================================    
//  AVATAR ANIMATION
// =============================================================================    
    
    public function animateTo (toX:Number, toY:Number):void {
      animator.setValues({x:toX, y:toY});
    }
    
    public function setPosition (x:Number, y:Number):void {
      this.x = x;
      this.y = y;
    }
    
    public function addPosition (position:Point):void {
      positions.addPosition(position);
      // If there were no positions in the queue before this call, manually
      // issue the request to move to the next position. Otherwise, the
      // queue is already being processed, and the move to the next position 
      // will be requested automatically.
      if (positions.length == 1) {
        nextPosition();
      }
    }
    
    public function addPositions (newPositions:PositionList):void {
      var numOldPositions:int = positions.length;
      var numNewPositions:int = newPositions.length;
      positions.addPositions(newPositions);
      
      // If there were no positions in the queue before this call, manually
      // issue the request to move to the next position. Otherwise, the
      // queue is already being processed, and the move to the next position 
      // will be requested automatically.
      if (numOldPositions == 0 && numNewPositions > 0) {
        nextPosition();
      }
    }

    protected function animationEndListener (tween:GTween):void {
      positions.shift();
      if (positions.length > 0) {
        nextPosition();
      }
    }
    
    protected function nextPosition ():void {
      var pos:Point = positions.getPositionAt(0);
      animateTo(pos.x, pos.y);
    }
    
// =============================================================================    
//  AVATAR POSITION LIST ATTRIBUTE MANAGEMENT
// =============================================================================    
    
    /**
     * Listener triggered when a client in this room updates an attribute.
     */
    protected function updateAttributeListener (e:AttributeEvent):void {
      var id:String = IClient(e.target).getClientID();
      var avatar:Avatar;
      var posList:PositionList = new PositionList();
      
      if (e.getChangedAttr().name == positionListAttributeName
          && e.getChangedAttr().scope == positionListAttributeScope) {
        posList.deserialize(e.getChangedAttr().value);
        // Update the user's current position.
        addPositions(posList);
      }
    }
    
// =============================================================================    
//  DISPOSAL
// =============================================================================    
    
    /**
     * Disposes of the resources used by this object, preparing it for garbage
     * collection.
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function dispose ():void {
      setClient(null);
    }
  }
}