package net.user1.reactor.ui {
  import flash.display.*;
  import flash.events.*;
  
  import net.user1.reactor.*;
  import net.user1.reactor.motion.PositionBroadcaster;
  import net.user1.utils.EventUtil;
  
  /**
   * A visual container that displays Avatar objects for a specified room.
   * For each client that joins the specified room and also defines an 
   * attribute named PositionBroadcaster.DEFAULT_POSITION_LIST_ATTRIBUTE_NAME
   * scoped to the room, the AvatarPane automatically adds a corresponding Avatar.
   * 
   * @since ReactorGUI 1.0.0 
   */ 
  public class AvatarPane extends Sprite {
// =============================================================================
// VARIABLES
// =============================================================================
    protected var room:Room;
    protected var avatars:Object;
    protected var defaultAvatarSpeed:int;
    protected var defaultAvatarIconClass:Class;
    protected var _hideSelf:Boolean;
    protected var _useCustomContainer:Boolean;

// =============================================================================
// CONSTRUCTOR
// =============================================================================
    /**
     * Constructor
     * 
     * @param defaultAvatarIconClass The icon to use for avatars in the room.
     * @param defaultAvatarSpeed The default speed for avatars in the room.
     * @param room The underlying Room object that will provide data for this
     *             AvatarPane.
     */
    public function AvatarPane (defaultAvatarIconClass:Class,
                                defaultAvatarSpeed:int = 100,
                                room:Room = null) {
      avatars = new Object();
      this.defaultAvatarSpeed     = defaultAvatarSpeed;
      this.defaultAvatarIconClass = defaultAvatarIconClass;
      setRoom(room);
    }
    
// =============================================================================
// VISUAL SETTINGS
// =============================================================================
    
    /**
     * Instructs this AvatarPane to keep the current client's Avatar object  
     * hidden.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function hideSelf ():void {
      _hideSelf = true;
      
      if (room != null) {
        for each (var occupant:IClient in room.getOccupants()) {
          if (occupant.isSelf()) {
            removeAvatar(occupant.getClientID());
          }
        }
      }
    }
    
    /**
     * Instructs this AvatarPane to allow the current client's Avatar object  
     * to be shown.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function showSelf ():void {
      _hideSelf = false;
      
      if (room != null) {
        for each (var occupant:IClient in room.getOccupants()) {
          if (occupant.isSelf()) {
            addAvatar(occupant.getClientID());
          }
        }
      }
    }
    
// =============================================================================
// DEPENDENCIES
// =============================================================================
    
    /**
     * Specifies the Room object for which this AvatarPane will provide
     * avatar visuals.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function setRoom (room:Room):void {
      EventUtil.migrateListeners(this.room,
                                 room,
                                 [RoomEvent.LEAVE, leaveListener,
                                  RoomEvent.REMOVE_OCCUPANT, removeClientListener,
                                  RoomEvent.ADD_OCCUPANT, addClientListener,
                                  RoomEvent.UPDATE_CLIENT_ATTRIBUTE, updateClientAttributeListener,
                                  RoomEvent.REMOVED, roomRemovedListener]);
      this.room = room;
      if (room != null) {
        refreshDisplay();
      } else {
        clearPane();
      }
    }

// =============================================================================
// ROOM EVENT LISTENERS
// =============================================================================

    /**
     * Listener triggered when the client leaves the room.
     */
    protected function leaveListener (e:RoomEvent):void {
      clearPane();
    }
    
    /**
     * Listener triggered when a client joins the room.
     */
    protected function addClientListener (e:RoomEvent):void {
      if (e.getClient().isSelf() && _hideSelf) {
        return;
      }
      // If the client has a POSITION_LIST attribute, treat it as an avatar
      var posListAttr:String = e.getClient().getAttribute(room.getRoomID(), PositionBroadcaster.DEFAULT_POSITION_LIST_ATTRIBUTE_NAME);
      if (posListAttr != null) {
        addAvatar(e.getClient().getClientID());
      }
    }
    
    /**
     * Listener triggered when a client in the room changes a client attribute.
     */
    protected function updateClientAttributeListener (e:RoomEvent):void {
      if (e.getClient().isSelf() && _hideSelf) {
        return;
      }
      
      // If the client previously had no avatar and has now defined a
      // position list attribute, add an avatar for the client.
      if (avatars[e.getClientID()] == null
        && e.getChangedAttr().name == PositionBroadcaster.DEFAULT_POSITION_LIST_ATTRIBUTE_NAME
        && e.getChangedAttr().scope == room.getRoomID()) {
        addAvatar(e.getClientID());
      }
    }
    
    /**
     * Listener triggered when a client leaves the room.
     */
    protected function removeClientListener (e:RoomEvent):void {
      // If the client has a position list attribute, treat it as an avatar
      var posListAttr:String = e.getClient().getAttribute(PositionBroadcaster.DEFAULT_POSITION_LIST_ATTRIBUTE_NAME, room.getRoomID());
      if (posListAttr != null) {
        removeAvatar(e.getClient().getClientID());
      }
    }
        
    /**
     * Listener triggered when the room is removed.
     */
    protected function roomRemovedListener (e:RoomEvent):void {
      clearPane();
      setRoom(null);
    }

// =============================================================================
// AVATAR MANAGEMENT
// =============================================================================

    /**
     * Adds a new avatar for the specified clientID. 
     */
    protected function addAvatar (clientID:String):Avatar {
      var avatar:Avatar;
      
      if (avatars[clientID] == null) {
        avatar = avatars[clientID] = new Avatar(defaultAvatarSpeed);
        avatar.setIcon(new defaultAvatarIconClass());
        avatar.setPositionListAttributeScope(room.getRoomID());
        avatar.setClient(room.getClient(clientID));
        if (!_useCustomContainer) {
          addChild(avatar);
        }
        fireAvatarAdded(avatar);
        return avatar;
      } else {
        return avatars[clientID];
      }
    }

    /**
     * Removes the specified avatar.
     */
    protected function removeAvatar (clientID:String):void {
      if (avatars[clientID] != null) {
        var removedAvatar:Avatar = avatars[clientID];
        if (!_useCustomContainer) {
          removeChild(removedAvatar);
        }
        delete avatars[clientID];
        fireAvatarRemoved(removedAvatar);
        removedAvatar.dispose();
      }
    }

    /**
     * Returns the Avatar object for the specified clientID.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function getAvatar (clientID:String):Avatar {
      return avatars[clientID];
    }

    /**
     * Returns an Array of all Avatar objects currently in this AvatarPane's
     * avatar list.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function getAllAvatars ():Array {
      var avatarList:Array = [];
      for each (var avatar:Avatar in avatars) {
        avatarList.push(avatar);
      }
      return avatarList;
    }

        
    /**
     * Removes all avatars from this pane.
     */
    protected function removeAllAvatars ():void {
      for (var avatarID:String in avatars) {
        removeAvatar(avatarID);
      }
    }

// =============================================================================
// DISPLAY MANAGEMENT
// =============================================================================
    
    /**
     * Determines whether this AvatarPane adds Avatar objects as its own 
     * display children. When set to false, this AvatarPane will not 
     * automatically add Avatar objects; instead, the application must
     * add Avatar objects to a DisplayObjectContainer of its choosing by
     * listening for the AvatarPaneEvent.ADD_AVATAR and
     * AvatarPaneEvent.REMOVE_AVATAR events.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function useCustomContainer (value:Boolean):void {
      _useCustomContainer = value;
    }

    /**
     * Synchronizes the avatars with the room's client list.
     */
    protected function refreshDisplay ():void {
      var posListAttr:String;
      var clientID:String;
      var avatar:Avatar;
      var clientIsGone:Boolean;
      
      for each (var client:Client in room.getOccupants()) {
        clientID = client.getClientID();
        posListAttr = client.getAttribute(room.getRoomID(), PositionBroadcaster.DEFAULT_POSITION_LIST_ATTRIBUTE_NAME);
        // If the client has a POSITION_LIST attribute, treat it as an avatar
        if (posListAttr != null) {
          avatar = avatars[clientID];
          // If there's no avatar for this client, make one
          if (avatar == null) {
            addAvatar(clientID);
          }
        }
      }
      
      // Remove avatars that are no longer in the client list
      for (var avatarID:String in avatars) {
        clientIsGone = true;
        for each (clientID in room.getOccupantIDs()) {
          if (avatarID == clientID) {
            clientIsGone = false;
            break;
          }
        }
        if (clientIsGone) {
          removeAvatar(clientID);
        }
      }
    }

    /**
     * Releases all pane resources.
     */
    protected function clearPane ():void {
      removeAllAvatars();
    }
    
// =============================================================================
// EVENT DISPATCHING
// =============================================================================
    
    protected function fireAvatarAdded (avatar:Avatar):void {
      dispatchEvent(new AvatarPaneEvent(AvatarPaneEvent.ADD_AVATAR, false, 
                                        false, avatar));
    }

    protected function fireAvatarRemoved (avatar:Avatar):void {
      dispatchEvent(new AvatarPaneEvent(AvatarPaneEvent.REMOVE_AVATAR, false, 
                                        false, avatar));
    }
  }
}