package net.user1.reactor.ui {
  import net.user1.reactor.IClient;
  import net.user1.reactor.MessageManager;
  import net.user1.reactor.Room;
  import net.user1.reactor.chat.ChatClient;
  import net.user1.reactor.chat.ChatMessages;
  import net.user1.utils.EventUtil;

  /**
   * A TextArea in which to display incoming messages in a chat interface.
   * Each ChatIncomingTextArea object must be connected to a Reactor Room
   * object and a Reactor MessageManager object. To add a regular chat message 
   * to a ChatIncomingTextArea, send a ChatMessages.CHAT message to
   * the specified room. For example,
   * 
   * <listing>
   * theRoom.sendMessage(ChatMessages.CHAT, "Hello world!");
   * </listing>
   * 
   * To add a private message to a ChatIncomingTextArea, send a
   * ChatMessages.PRIVATE_CHAT message to a specific client. For example,
   * 
   * <listing>
   * theClient.sendMessage(ChatMessages.PRIVATE_CHAT, "How old are you?");
   * </listing>
   * 
   * @since ReactorGUI 1.0.0
   */
  public class ChatIncomingTextArea extends AutoScrollTextArea {
// =============================================================================
// VARIABLES
// =============================================================================
    protected var _room:Room;
    protected var _messageManager:MessageManager;
    
// =============================================================================
// CONSTRUCTOR
// =============================================================================
    /**
     * Constructor
     */
    public function ChatIncomingTextArea () {
      editable = false;
    }
    
// =============================================================================
// DEPENDENCIES
// =============================================================================

    /**
     * @private
     */
    public function set messageManager (value:MessageManager):void {
      if (_messageManager == value) {
        return;
      }
      if (_messageManager != null) {
        _messageManager.removeMessageListener(ChatMessages.PRIVATE_CHAT, privateChatListener);
      }
      _messageManager = value;
      if (_messageManager != null) {
        _messageManager.addMessageListener(ChatMessages.PRIVATE_CHAT, privateChatListener);
      }
      
      _messageManager = value;
    }
    
    /**
     * A Reactor MessageManager reference used to receive private chat messages.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get messageManager ():MessageManager {
      return _messageManager;
    }
    
    /**
     * @private
     */
    public function set room (value:Room):void {
      if (_room == value) {
        return;
      }
      if (_room != null) {
        _room.removeMessageListener(ChatMessages.CHAT, chatListener);
      }
      _room = value;
      if (_room != null) {
        _room.addMessageListener(ChatMessages.CHAT, chatListener);
      }
    }
    
    /**
     * A Reactor Room reference used to receive chat messages.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get room ():Room {
      return _room;
    }
    
// =============================================================================
// CHAT EVENT LISTENERS
// =============================================================================

    protected function chatListener (fromClient:IClient, message:String):void {
      displayMessage(ChatClient(fromClient).getUserName() + ": " + message);
    }

    protected function privateChatListener (fromClient:IClient, message:String):void {
      displayMessage("[Private] " + ChatClient(fromClient).getUserName() + ": " + message);
    }

// =============================================================================
// UI MANAGEMENT
// =============================================================================

    /**
     * Displays a line of text in the incoming chat text field.
     */
    public function displayMessage (message:String):void {
      // Add the message to the incoming chat text field
      appendText(message + "\n");
    }
    
// =============================================================================
// DISPOSE
// =============================================================================
    
    /**
     * Disposes of the resources used by this object, preparing it for garbage
     * collection.
     * 
     * @since ReactorGUI 1.0.0
     */    
    override public function dispose ():void {
      super.dispose();
      room = null;
    }
  }
}