package net.user1.reactor.ui {
  import flash.events.KeyboardEvent;
  import flash.events.MouseEvent;
  import flash.ui.Keyboard;
  
  import mx.containers.ControlBar;
  import mx.containers.HBox;
  import mx.containers.Panel;
  import mx.containers.VBox;
  import mx.controls.CheckBox;
  import mx.controls.LinkButton;
  import mx.controls.Spacer;
  import mx.controls.TextInput;
  import mx.events.CollectionEvent;
  import mx.events.ListEvent;
  import mx.managers.PopUpManager;
  
  import net.user1.reactor.AttributeEvent;
  import net.user1.reactor.IClient;
  import net.user1.reactor.Room;
  import net.user1.reactor.RoomEvent;
  import net.user1.reactor.chat.ChatAttributes;
  import net.user1.reactor.chat.ChatClient;
  import net.user1.reactor.chat.ChatMessages;
  import net.user1.utils.EventUtil;
  
  /**
   * A panel containing a chat user interface, including a user list,
   * private messaging, incoming and outgoing chat messages, and a chat topic.
   * 
   * @since ReactorGUI 1.0.0
   */ 
  public class ChatPanel extends Panel {
// =============================================================================
// VARIABLES
// =============================================================================
    // Containers
    private var topBox:HBox;
    private var bottomBox:HBox;
    private var userBox:VBox;
    private var bottomBar:ControlBar;
    
    // Controls
    private var _userlist:RoomOccupantList;
    private var privateCheckBox:CheckBox;
    private var incoming:ChatIncomingTextArea;
    private var outgoing:TextInput;
    private var sendButton:ChatSendButton;
    private var _accountBar:AccountBar;
    private var changetopicButton:LinkButton;
    
    private var _room:Room;
    
// =============================================================================
// CONSTRUCTOR
// =============================================================================
    public function ChatPanel () {
      buildUI();
    }
    
    protected function buildUI ():void {
      setStyle("paddingLeft",   3);
      setStyle("paddingRight",  3);
      setStyle("paddingBottom", 3);
      setStyle("paddingTop",    3);

      // Create child components
      topBox = new HBox();
      topBox.percentWidth = 100;
      topBox.percentHeight = 100;
      
      bottomBox = new HBox();
      bottomBox.percentWidth = 100;
      
      userBox = new VBox();
      userBox.percentWidth = 30;
      userBox.percentHeight = 100;
      
      _userlist = new RoomOccupantList();
      _userlist.addEventListener(ListEvent.CHANGE, userlistSelectionChangeListener);
      _userlist.dataProvider.addEventListener(CollectionEvent.COLLECTION_CHANGE, userlistProviderChangeListener)
      _userlist.percentWidth = 100;
      _userlist.percentHeight = 100;
      
      privateCheckBox = new CheckBox();
      privateCheckBox.label = "Private Chat";
      privateCheckBox.addEventListener(MouseEvent.CLICK, privateCheckBoxClickListener);
      refreshPrivateCheckbox();

      incoming = new ChatIncomingTextArea();
      incoming.percentHeight = 100;
      incoming.percentWidth = 100;
      
      outgoing = new TextInput();
      outgoing.percentWidth = 100;
      outgoing.addEventListener(KeyboardEvent.KEY_DOWN, outgoingKeyListener);

      sendButton = new ChatSendButton();
      sendButton.addEventListener(MouseEvent.CLICK, sendClickListener);
      
      // Assemble child components
      userBox.addChild(_userlist);
      userBox.addChild(privateCheckBox);
      
      topBox.addChild(incoming);
      topBox.addChild(userBox);

      bottomBox.addChild(outgoing);
      bottomBox.addChild(sendButton);
      
      changetopicButton = new LinkButton();
      changetopicButton.label = "Change Topic";
      changetopicButton.addEventListener(MouseEvent.CLICK, changeTopicClickListener);
      
      var spacer:Spacer = new Spacer();
      spacer.percentWidth = 100;
      
      _accountBar = new AccountBar();
      
      bottomBar = new ControlBar();
      bottomBar.addChild(changetopicButton);
      bottomBar.addChild(spacer);
      bottomBar.addChild(_accountBar);
      
      addChild(topBox);
      addChild(bottomBox);
      addChild(bottomBar);
    }

// =============================================================================    
//  DEPENDENCIES  
// =============================================================================    
    
    /** 
     * @private
     */
    public function set room (value:Room):void {
      EventUtil.migrateListeners(_room, value,
               [AttributeEvent.UPDATE, updateRoomAttributeListener,
                RoomEvent.ADD_OCCUPANT, addClientListener,
                RoomEvent.REMOVE_OCCUPANT, removeClientListener,
                RoomEvent.SYNCHRONIZE, synchronizeRoomListener]);
      _room = value;
      _userlist.room = _room;
      incoming.room = _room;

      refreshTitle();
      refreshSendButton();
    }
    
    /** 
     * A reference to the Reactor Room object for this chat panel. Users that
     * join the room are considered participants in the chat, and appear in
     * the chat user list.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get room ():Room {
      return _room;
    }
    
// =============================================================================
// ROOM EVENT LISTENERS
// =============================================================================

    /**
     * Triggered when a room attribute changes.
     */
    protected function updateRoomAttributeListener (e:AttributeEvent):void {
      var byClient:ChatClient = ChatClient(e.getChangedAttr().byClient);

      if (e.getChangedAttr().name == ChatAttributes.ROOM_TOPIC) {
        refreshTitle();
        if (byClient != null) {
          incoming.displayMessage(byClient.getUserName() 
                                + " set the topic to: " + e.getChangedAttr().value + ".");
        }
      }
    }

    /**
     * Triggered when a client joins the room.
     */
    protected function addClientListener (e:RoomEvent):void {
      refreshTitle();
    }

    /**
     * Triggered when a client leaves the room.
     */
    protected function removeClientListener (e:RoomEvent):void {
      if (e.getClient() == sendButton.client) {
        sendButton.client = null;
      }
      refreshTitle();
    }

    /**
     * Triggered when the local room has been synchronized with the server.
     */
    protected function synchronizeRoomListener (e:RoomEvent):void {
      refreshTitle();
    }
    
// =============================================================================
// CHAT COMMANDS
// =============================================================================

    /**
     * Sends a chat message to all clients in the room.
     */
    public function sendChatMessage ():void {
      // Can't send a message without a room
      if (room == null) {
        return;
      }
      
      // If the "message" field has something to send
      if (outgoing.length > 0) {
        if (privateCheckBox.selected) {
          // If it's a private message
          if (_userlist.selectedItem != null) {
            // If the selected user is the current user
            if (ChatClient(_userlist.selectedItem.data).isSelf()) {
              // Display warning: can't send a message to yourself
              incoming.displayMessage("You told yourself: " + outgoing.text);
            } else {
              // Send private message to selected user
              incoming.displayMessage("You told " 
                              + _userlist.selectedItem.label + ": " + outgoing.text); 
              ChatClient(_userlist.selectedItem.data).sendPrivateMessage(outgoing.text);
            }
          } else {
            incoming.displayMessage("Please select a user before sending your private message.");
            return; 
          }
        } else {
           // ...send the the message to all occupants of the room
          _room.sendMessage(ChatMessages.CHAT, true, null, outgoing.text);
        }

        // Clear the "message" field
        outgoing.text = "";
      }
    }

// =============================================================================
// UI EVENT LISTENERS
// =============================================================================

   /**
    * Listener triggered when the "Change Topic" button is clicked.
    */
    protected function changeTopicClickListener (e:MouseEvent):void {
      var popup:TextInputWindow = TextInputWindow(
                          PopUpManager.createPopUp(this, TextInputWindow, true));
      popup.title = "Set New Topic";
      popup.addEventListener(TextInputWindowEvent.SUBMIT, submitListener);      
      PopUpManager.centerPopUp(popup);

      function submitListener (e:TextInputWindowEvent):void {
        if (e.getInput().length > 0) {
          if (_room != null) {
            _room.setAttribute(ChatAttributes.ROOM_TOPIC, e.getInput());
          }
        }
        popup.close();
      }
    }

   /**
    * Listener triggered when the "Private Chat" checkbox is clicked.
    */
    protected function privateCheckBoxClickListener (e:MouseEvent):void {
      refreshSendButton();
    }

   /**
    * Listener triggered when the user list selection changes.
    */
    protected function userlistSelectionChangeListener (e:ListEvent):void {
      refreshPrivateCheckbox();
      refreshSendButton();
    }

   /**
    * Listener triggered when the user list selection changes.
    */
    protected function userlistProviderChangeListener (e:CollectionEvent):void {
      refreshPrivateCheckbox();
    }

   /**
    * Listener triggered when the "Send" button is clicked.
    */
    protected function sendClickListener (e:MouseEvent):void {
      sendChatMessage();
    }

    /**
     * Listener triggered whenever a key is pressed while the "outField" has
     * focus. This listener lets the user use the ENTER key to send messsages.
     */ 
    private function outgoingKeyListener (e:KeyboardEvent):void {
      if (e.keyCode == Keyboard.ENTER) {
        sendChatMessage();
      }
    }

// =============================================================================
// UI MANAGEMENT
// =============================================================================

    protected function refreshTitle ():void {
      var roomTitle:String;
      
      if (_room != null) {
        roomTitle = room.getAttribute(ChatAttributes.ROOM_TOPIC);
        
        if (roomTitle != null) {
          title = roomTitle + " (" + _room.getNumOccupants() + " chatting)";
        } else {
          title = "(" + _room.getNumOccupants() + " chatting)";
        }
      } else {
        // Null room
        title = "Chat inactive";
      }
    }

    protected function refreshSendButton ():void {
      sendButton.enabled = room != null;
      
      if (privateCheckBox.selected) {
        if (_userlist.selectedItem == null) {
          sendButton.client = null;
        } else {
          sendButton.client = _userlist.selectedItem.data;
        }
      } else {
        sendButton.client = null;
      }
    }
    
    protected function refreshPrivateCheckbox ():void {
      if (_userlist.selectedItem == null) {
        privateCheckBox.enabled = false;
      } else {
        privateCheckBox.enabled = true;
      }
    }

// =============================================================================
// UI ACCESS
// =============================================================================
    
    /**
     * The AccountBar object contained by this ChatPanel.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get accountBar ():AccountBar {
      return _accountBar;
    }
    
    /**
     * The UserList object contained by this ChatPanel.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get userlist ():RoomOccupantList {
      return _userlist;
    }
    
    /**
     * The ChatIncomingTextArea object contained by this ChatPanel.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get chatIncomingTextArea ():ChatIncomingTextArea {
      return incoming;
    }
  }
}