package net.user1.reactor.ui {
  import mx.containers.HBox;
  import mx.containers.Panel;
  
  import net.user1.reactor.AttributeEvent;
  import net.user1.reactor.Room;
  import net.user1.reactor.RoomEvent;
  import net.user1.reactor.chat.ChatAttributes;
  import net.user1.utils.EventUtil;
  
  /**
   * A panel containing a spectator interface for a chat, as might be used by
   * chat moderators or administrators. Includes a user list and a text field
   * displaying incoming chat messages.
   * 
   * @since ReactorGUI 1.0.0
   * 
   * @see ChatPanel
   */ 
  public class ChatSpectatorPanel extends Panel {
// =============================================================================
// VARIABLES
// =============================================================================
    // Containers
    private var topBox:HBox;
    
    // Controls
    private var _userlist:RoomOccupantList;
    private var incoming:ChatIncomingTextArea;
    
    private var _room:Room;
    
// =============================================================================
// CONSTRUCTOR
// =============================================================================    
    public function ChatSpectatorPanel () {
      buildUI();
    }
    
    protected function buildUI ():void {
      setStyle("paddingLeft",   3);
      setStyle("paddingRight",  3);
      setStyle("paddingBottom", 3);
      setStyle("paddingTop",    3);

      // Create child components
      topBox = new HBox();
      topBox.percentWidth = 100;
      topBox.percentHeight = 100;
      
      _userlist = new RoomOccupantList();
      _userlist.percentWidth = 30;
      _userlist.percentHeight = 100;

      incoming = new ChatIncomingTextArea();
      incoming.percentHeight = 100;
      incoming.percentWidth = 100;
      
      // Assemble child components
      topBox.addChild(incoming);
      topBox.addChild(_userlist);

      addChild(topBox);
    }
    
// =============================================================================
// DEPENDENCIES
// =============================================================================    

    /**
     * @private
     */
    public function set room (value:Room):void {
      EventUtil.migrateListeners(_room, value,
                  [AttributeEvent.UPDATE, updateRoomAttributeListener,
                   RoomEvent.ADD_OCCUPANT, addClientListener,
                   RoomEvent.REMOVE_OCCUPANT, removeClientListener,
                   RoomEvent.SYNCHRONIZE, synchronizeRoomListener]);
      
      _room = value;
      _userlist.room = _room;
      incoming.room = _room;
      refreshTitle();
    }
    
    /** 
     * A reference to the Reactor Room object for this chat panel. Users that
     * join the room are considered participants in the chat, and appear in
     * the chat user list.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get room ():Room {
      return _room;
    }

// =============================================================================
// ROOM EVENT LISTENERS
// =============================================================================

    /**
     * Triggered when a room attribute changes.
     */
    protected function updateRoomAttributeListener (e:AttributeEvent):void {
      if (e.getChangedAttr().name == ChatAttributes.ROOM_TOPIC) {
        refreshTitle();
      }
    }

    /**
     * Triggered when a client joins the room.
     */
    protected function addClientListener (e:RoomEvent):void {
      refreshTitle();
    }

    /**
     * Triggered when a client leaves the room.
     */
    protected function removeClientListener (e:RoomEvent):void {
      refreshTitle();
    }

    /**
     * Triggered when the local room has been synchronized with the server.
     */
    protected function synchronizeRoomListener (e:RoomEvent):void {
      refreshTitle();
    }

// =============================================================================
// UI MANAGEMENT
// =============================================================================

    protected function refreshTitle ():void {
      var roomTitle:String;
      
      if (_room != null) {
        roomTitle = room.getAttribute(ChatAttributes.ROOM_TOPIC);

        if (roomTitle != null) {
          title = roomTitle + " (" + _room.getNumOccupants() + " chatting)";
        } else {
          title = "(" + _room.getNumOccupants() + " chatting)";
        }
      } else {
        // Null room
        title = "Chat inactive";
      }
    }

// =============================================================================
// UI ACCESS
// =============================================================================
    
    /**
     * The UserList object contained by this ChatPanel.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get userlist ():RoomOccupantList {
      return _userlist;
    }
  }
}