package net.user1.reactor.ui {
  import mx.collections.ArrayCollection;
  import mx.collections.Sort;
  import mx.collections.SortField;
  import mx.controls.DataGrid;
  import mx.controls.dataGridClasses.DataGridColumn;
  
  import net.user1.reactor.AttributeEvent;
  import net.user1.reactor.ClientEvent;
  import net.user1.reactor.IClient;
  
  /**
   * A data grid that displays a client's attributes. Used in
   * administrative and debugging interfaces.
   * 
   * @since 1.0.0
   *
   * @see RoomAttributeGrid
   */
  public class ClientAttributeGrid extends DataGrid {
// =============================================================================
// VARIABLES
// =============================================================================  
    private var _client:IClient;
    
// =============================================================================
// CONSTRUCTOR
// =============================================================================  
    public function ClientAttributeGrid () {
      dataProvider = new ArrayCollection();
      var nameColumn:DataGridColumn = new DataGridColumn("Name");
      nameColumn.dataField = "name";
      nameColumn.width = .4;
      var valueColumn:DataGridColumn = new DataGridColumn("Value");
      valueColumn.dataField = "value";
      valueColumn.width = .6;
      columns = [nameColumn, valueColumn];
      
      var sort:Sort = new Sort();
      sort.fields = [new SortField("name",true)];
      dataProvider.sort = sort;
    }
    
// =============================================================================
// DEPENDENCIES
// =============================================================================  

    /**
     * @private
     */
    public function set client (newClient:IClient):void {
      
      if (_client != null) {
        _client.removeEventListener(AttributeEvent.UPDATE, updateAttributeListener);
        _client.removeEventListener(AttributeEvent.DELETE, deleteAttributeListener);
      }
      
      _client = newClient;
      
      if (newClient != null) {
        _client.addEventListener(AttributeEvent.UPDATE, updateAttributeListener);
        _client.addEventListener(AttributeEvent.DELETE, deleteAttributeListener);
      }
      
      clearList();
      refreshDisplay();
    }
    
    /**
     * The Client whose attributes will be displayed by this ClientAttributeGrid.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get client ():IClient {
      return _client;
    }  
    
    /**
     * @private
     */
    protected function updateAttributeListener (e:AttributeEvent):void {
      refreshDisplay();
    }
    
    /**
     * @private
     */
    protected function deleteAttributeListener (e:AttributeEvent):void {
      refreshDisplay();
    }
    
// =============================================================================
// UI MANAGEMENT
// =============================================================================
    
    /**
     * Updates the data grid to reflect all attributes defined by the target
     * client.
     */
    protected function refreshDisplay ():void {
      var attrExists:Boolean;
      var attrs:Object;
      
      if (_client != null) {
        attrs = _client.getAttributes();
        
        // Add all new attrs to the provider
        for (var attrName:String in attrs) {
          attrExists = false;
          for (var i:int = dataProvider.length; --i >= 0;) {
            if (dataProvider.getItemAt(i).name == attrName) {
              attrExists = true;
              break;
            }
          }
          if (!attrExists) {
            addAttributeToGrid(attrName, attrs[attrName]);
          } else {
            dataProvider.getItemAt(i).value = attrs[attrName];
            dataProvider.itemUpdated(dataProvider.getItemAt(i));
          }
        }
        
        // Remove all missing attrs from the provider
        for (i = dataProvider.length; --i >= 0;) {
          attrExists = false;
          for (attrName in attrs) {
            if (attrName == dataProvider.getItemAt(i).name) {
              attrExists = true;
              break;
            }
          }
          if (!attrExists) {
            dataProvider.removeItemAt(i);
          }
        }
      }
    }
    
    /**
     * Adds a new attribute to the data grid.
     */
    protected function addAttributeToGrid (attrName:String, attrValue:String):void {
      dataProvider.addItem({name:attrName, value:attrValue});
      dataProvider.refresh();
    }
    
    /**
     * Removes all content from the attribute list.
     */
    protected function clearList ():void {
      dataProvider.removeAll();
    }
    
// =============================================================================
// DISPOSAL
// =============================================================================
    
    /**
     * Disposes of the resources used by this object, preparing it for garbage
     * collection.
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function dispose ():void {
      if (_client != null) {
        _client.removeEventListener(AttributeEvent.UPDATE, updateAttributeListener);
        _client.removeEventListener(AttributeEvent.DELETE, deleteAttributeListener);
      }
      _client = null;
      dataProvider = null;
    }  
  }
}