package net.user1.reactor.ui {
  import flash.events.TimerEvent;
  import flash.utils.Timer;
  
  import mx.collections.ArrayCollection;
  import mx.collections.Sort;
  import mx.collections.SortField;
  import mx.controls.DataGrid;
  import mx.controls.dataGridClasses.DataGridColumn;
  import mx.controls.dataGridClasses.DataGridItemRenderer;
  import mx.events.FlexEvent;
  
  import net.user1.reactor.Client;
  import net.user1.reactor.ClientManager;
  import net.user1.reactor.ClientManagerEvent;
  import net.user1.reactor.IClient;
  import net.user1.reactor.UserAccount;
  import net.user1.utils.EventUtil;
  
  /**
   * Displays a list of clients currently known to Reactor. To display all
   * clients on the server, first invoke ClientManager's watchForClients()
   * method.
   * 
   * @since ReactorGUI 1.0.0
   */  
  public class ClientGrid extends DataGrid {
    protected var _clientManager:ClientManager;
    private var refreshTimer:Timer;
    
    protected const stateDescriptions:Array = ["Unknown",
                                               "Not connected",
                                               "Ready",
                                               "Connection in progress",
                                               "Disconnection in progress",
                                               "Logged in",
                                               "Deleted"];

    public function ClientGrid () {
      dataProvider = new ArrayCollection();
      var clientColumn:DataGridColumn = new DataGridColumn();
      clientColumn.dataField = "clientIDLabel";
      clientColumn.headerText = "Client ID";
      clientColumn.width = 60;

      var userColumn:DataGridColumn = new DataGridColumn();
      userColumn.dataField = "userID";
      userColumn.headerText = "User Account";
      var stateColumn:DataGridColumn = new DataGridColumn();
      stateColumn.dataField = "state";
      stateColumn.headerText = "State";
      stateColumn.width = 80;
      
      columns = [clientColumn, userColumn, stateColumn];
      
      refreshTimer = new Timer(1000);
      refreshTimer.addEventListener(TimerEvent.TIMER, refreshTimerListener);
      refreshTimer.start();
      
      var sort:Sort = new Sort();
      sort.fields = [new SortField("clientID", true, false, true)];
      dataProvider.sort = sort;
    }
    
// =============================================================================
// REFRESH TIMER
// =============================================================================

    protected function refreshTimerListener (e:TimerEvent):void {
      refreshClientDetails();
    }
    
    public function get clientManager():ClientManager {
      return _clientManager;
    }

    public function set clientManager(value:ClientManager):void {
      EventUtil.migrateListeners(_clientManager, value, 
                                 [ClientManagerEvent.CLIENT_CONNECTED, clientConnectedListener,
                                  ClientManagerEvent.CLIENT_DISCONNECTED, clientDisconnectedListener]);
      _clientManager = value;
      clearProvider();
      populateProvider();
    }
    
    protected function populateProvider ():void {
      if (_clientManager != null) {
        var account:UserAccount;
        for each (var client:Client in _clientManager.getInternalClients()) {
          account = client.getAccount();
          dataProvider.source.push({clientID:parseInt(client.getClientID()),
                                    clientIDLabel:client.getClientID() + (client.isSelf() ? " [YOU]" : ""), 
                                    userID:(account != null ? account.getUserID() : "--"),
                                    state:stateDescriptions[client.getConnectionState()+1],
                                    client:client});
        }
        dataProvider.refresh();
      }
    }

    protected function clientConnectedListener (e:ClientManagerEvent):void {
      var client:IClient = e.getClient();
      if (client.isSelf()) {
        // The current client is already known, so ignore any connection
        // event reported by watchForClients().
        return;
      }
      var account:UserAccount = client.getAccount();
      addItemToGrid(client.getClientID(),
                    client.getClientID() + (client.isSelf() ? " [YOU]" : ""), 
                    (account != null ? account.getUserID() : "--"),
                    stateDescriptions[client.getConnectionState()+1],
                    client);
    }
    
    protected function clientDisconnectedListener (e:ClientManagerEvent):void {
      removeItemFromGrid(e.getClientID());
    }
    
    protected function addItemToGrid (clientID:String, clientIDLabel:String, userID:String, state:String, client:IClient):void {
      dataProvider.source.push({clientID:clientID, clientIDLabel:clientIDLabel, userID:userID, state:state, client:client});
    }    
    
    protected function removeItemFromGrid (clientID:String):void {
      var item:Object;
      var length:int = dataProvider.source.length;
      var source:Array = dataProvider.source; 
      for (var i:int = length; --i >= 0;) {
        item = source[i];
        if (clientID == item.clientID) {
          source.splice(i, 1);
          return;
        }
      }
    }
    
    protected function refreshClientDetails ():void {
      var item:Object;
      var client:IClient;
      var account:UserAccount;
      var selfItem:Object;
      
      // For performance, update only the visible rows
      for each (var renderer:DataGridItemRenderer in visibleData) {
        item          = renderer.data;
        client        = item.client;
        account            = client.getAccount();
        item.state         = stateDescriptions[client.getConnectionState()+1];
        item.userID        = (account != null ? account.getUserID() : "--");
      }
      
      // Record grid state before refreshing provider
      var vPos:int = verticalScrollPosition;
      var oldSelectedIndex:int = selectedIndex;
      
      // Refresh provider
      dataProvider.refresh();
      
      // If selected item is now gone, the selection will have changed, in which
      // case, manually dispatch VALUE_COMMIT because the item was removed without
      // the provider's knowledge.
      if (oldSelectedIndex != selectedIndex) {
        dispatchEvent(new FlexEvent(FlexEvent.VALUE_COMMIT));
      }
      
      // Restore scroll position
      verticalScrollPosition = vPos;
    }
    
    protected function clearProvider ():void {
      dataProvider.source = new Array();
    }
  }
}