package net.user1.reactor.ui {
  import flash.display.Sprite;
  import flash.events.Event;
  import flash.events.MouseEvent;
  import flash.system.System;
  import flash.text.TextField;
  import flash.text.TextFormat;
  
  /**
   * <p>
   * A high-performance text field for displaying high volumes of text, such as
   * the contents of a debugging log. FastTextField has the following features:
   * </p>
   * 
   * <ul>
   * <li>can smoothly scroll hundreds-of-thousands of lines of text</li>
   * <li>allows tens-of-thousands of lines to be added per second</li>
   * <li>has a configurable "maximum lines" limit that allows for an infinite 
   * stream of text at a fixed memory cost.</li>
   * </ul>
   * 
   * <p>
   * For the sake of performance, FastTextField does not support word 
   * wrapping, per-character font formatting, nor most other styling features 
   * of regular text fields. All text in FastTextField must have the same format. 
   * However, each line of text can have its own color (see 
   * appendText() and setTextFormat()).
   * </p>
   * 
   * <p>
   * FastTextField does not include scrollbars; for a high-performance text area
   * with scrollbars, see FastTextArea.
   * </p>
   * 
   * @since ReactorGUI 1.0.0
   * 
   * @see FastTextArea
   */
  public class FastTextField extends Sprite {
// =============================================================================
// VARIABLES
// =============================================================================
    
    protected var textfield:TextField;

    protected var bottomLineIndex:int;
    protected var lines:Array;
    protected var dirtyText:Boolean;
    protected var dirtyCanvas:Boolean;
    
    protected var _width:int;
    protected var _height:int;
    protected var _borderColor:uint;
    protected var _borderAlpha:Number;
    protected var _borderThickness:int;
    
    protected var _backgroundColor:uint;
    protected var _backgroundAlpha:Number;
    protected var _scrollV:int;
    protected var _autoScroll:Boolean;
    protected var newformat:TextFormat;
    
    protected var _maxLength:int;
    protected var lineshead:int;
    
// =============================================================================
// CONSTRUCTOR
// =============================================================================
    
    public function FastTextField () {
      // Init vars
      bottomLineIndex = int.MAX_VALUE;
      lines = [];
      dirtyText = true;
      dirtyCanvas  = true;
      _width = 500;
      _height = 200;
      _borderAlpha = 1;
      _backgroundColor = 0xFFFFFF;
      _backgroundAlpha = 1;
      _scrollV = 1;
      _maxLength = 100000;
      newformat = new TextFormat();
      
      // Enable mouse events
      mouseEnabled = true;
      
      // Make the output textfield
      textfield = new TextField();
      addChild(textfield);
      setTextFormat(new TextFormat("_typewriter", 11));
      
      // Check for updates every frame and on mouse wheel
      this.addEventListener(Event.ENTER_FRAME, enterFrameListener);
      this.addEventListener(MouseEvent.MOUSE_WHEEL, mouseWheelListener);
    }

// =============================================================================
// EVENT LISTENERS
// =============================================================================
    
    // Check for redraw every frame
    protected function enterFrameListener (e:Event):void {
      // Don't draw if invisible
      if (stage == null || visible == false) {
        return;
      } else {
        if (dirtyCanvas) {
          drawChrome();
        }
        if (dirtyText) {
          drawText();
          
          if (_autoScroll) {
            scrollV = maxScrollV;
          }
        }      
      }
    }

    // Scroll on mouse wheel
    protected function mouseWheelListener (e:MouseEvent):void {
      scrollV -= e.delta;
    }
    
// =============================================================================
// RENDERING
// =============================================================================
    
    // Draw border and background
    protected function drawChrome ():void {
      // Draw the border
      graphics.clear();
      graphics.beginFill(_borderColor, _borderAlpha);
      graphics.drawRect(0, 0, _width, _height);
      graphics.drawRect(_borderThickness, _borderThickness, _width-_borderThickness*2, _height-_borderThickness*2);
      graphics.endFill();
      
      // Draw the background
      graphics.beginFill(_backgroundColor, _backgroundAlpha);
      graphics.drawRect(_borderThickness, _borderThickness, _width-_borderThickness*2, _height-_borderThickness*2);
      graphics.endFill();
      
      // Resize display surface.
      textfield.x = _borderThickness;
      textfield.y = _borderThickness;
      textfield.width = _width - _borderThickness*2;
      textfield.height = _height - _borderThickness*2;
      
      // If the text area is now taller, scroll up to fill empty space at 
      // the bottom if necessary
      if (scrollV > maxScrollV) {
        scrollV = maxScrollV;
      }
      
      // Layout changed so re-draw the text
      dirtyCanvas  = false;
      dirtyText = true;
    }
    
    // Draw text
    protected function drawText ():void {
      dirtyText = false;
      
      // Find visible range
      var startLine:int  = scrollV-1;
      var endLine:int    = Math.min(startLine + pageLineCount, lines.length);
      var numLinesToDraw:int = endLine - startLine;
      // Adjust start index within the circular array
      var adjustedStartLine:int = lineshead+1 + startLine;
      adjustedStartLine = adjustedStartLine > lines.length ? adjustedStartLine - lines.length : adjustedStartLine;
      
      // Clear text field and remember horizontal scroll position
      var oldScrollH:int = textfield.scrollH;
      textfield.text = "";
      
      // Draw lines to text field
      var line:TextLine;
      for (var i:int = adjustedStartLine, j:int = 0; j < numLinesToDraw; i++, j++) {
        if (i > lines.length-1) {
          i = 0;
        }
        line = lines[i];
        textfield.appendText(line.text + "\n");
        newformat.color = line.color;
        textfield.setTextFormat(newformat,
                                textfield.length - line.text.length - 1, 
                                textfield.length);
      }
      
      // Restore horizontal scroll position
      textfield.scrollH = oldScrollH;
    }

// =============================================================================
// RAW TEXT DATA MANAGEMENT
// =============================================================================
    
  /**
   * <p>
   * Replaces all text in the text field with the supplied value.
   * </p>
   * 
   * @since ReactorGUI 1.0.0
   * 
   * @see appendText()
   */
    public function set text (text:String):void {
      lines = [];
      textfield.text = "";
      appendText(text, null);
    }
    
  /**
   * <p>
   * Adds the specified text to the end of existing text in the text field. If
   * color is specified, the color of the new text and all preceding text on
   * the same line will be set to the specified color.
   * </p>
   * 
   * @since ReactorGUI 1.0.0
   * 
   * @see appendText()
   */
    public function appendText (text:String="", color:Object=null):void {
      // Replace all forms of line breaks with a standard line feed.
      text = text.replace(/\r\n/g, "\n");
      var endOfLine:String = String.fromCharCode(10);
      var endOfLineIndex:int = text.indexOf(endOfLine);
      
      // If the text field is currently empty, seed it with a new line
      if (lines.length == 0) {
        addNewLine("", color);
      }
      
      // If the only new line is at the end of the supplied text...
      if (endOfLineIndex > -1 && endOfLineIndex == text.length-1) {
        // ...append the text to the current line, then add a new empty line
        addToLastLine(text.substring(0, text.length-1), color);
        addNewLine("", color);
      // If there's a new line in the middle of the supplied text...
      } else if (endOfLineIndex != -1) {
        // ...split the supplied text into lines, then add each line individually
        var newlines:Array = text.split(endOfLine);
        addToLastLine(newlines[0], color);
        for (var i:int = 1; i < newlines.length; i++) {
          addNewLine(newlines[i], color);
        }
      // If there's a no new line in the supplied text...
      } else {
        // ...append the text to the current line
        addToLastLine(text);
      }
    }

    protected function addNewLine (text:String="", color:Object=null):void {
      if (lines.length < _maxLength) {
        lines.push(new TextLine(text, color));
        lineshead = lines.length-1;
      } else {
        // Wrap around
        lineshead = lineshead < _maxLength-1 ? lineshead+1 : 0;
        lines[lineshead].text  = text;
        lines[lineshead].color = color;
      }
      dirtyText = true;
    }

    protected function addToLastLine (text:String="", color:Object = null):void {
      var line:TextLine = lines[lineshead];
      line.text += text;
      if (color != null) {
        line.color = color;
      }
      dirtyText = true;
    }

// =============================================================================
// FORMATTING
// =============================================================================
    
    /**
     * <p>
     * Sets the font format of all text in the text.
     * </p>
     * 
     * @param format A TextFormat object that specifies formatting settings for
     *               text in the FastTextField. For the sake of performance,
     *               FastTextField supports a limited set of formatting options 
     *               only, as follows:
     * 
     * <ul> 
     * <li>font</li>
     * <li>size</li>
     * <li>color</li>
     * <li>bold</li>
     * <li>italics</li>
     * <li>leading</li>
     * <li>letterSpacing</li>
     * </ul>
     * 
     * @since ReactorGUI 1.0.0
     * 
     * @see appendText()
     */
    public function setTextFormat (format:TextFormat):void {
      var oldFormat:TextFormat = textfield.getTextFormat();
      var newFormat:TextFormat = new TextFormat();

      if (format.font)   newFormat.font                 = format.font;
      if (format.size)   newFormat.size                 = format.size;
      if (format.color)  newFormat.color                = format.color;
      if (format.bold)   newFormat.bold                 = format.bold;
      if (format.italic) newFormat.italic               = format.italic;
      if (format.leading) newFormat.leading             = format.leading;
      if (format.letterSpacing) newFormat.letterSpacing = format.letterSpacing;
      
      textfield.setTextFormat(newFormat);
      textfield.defaultTextFormat = newFormat;
    }

// =============================================================================
// CLIPBOARD
// =============================================================================
    
    
    /**
     * <p>
     * Copies all of the text field's text ( including any text that is
     * scrolled out of view) to the system clipboard.
     * </p>
     * 
     * @since ReactorGUI 1.0.0
     */
    public function copyToClipboard ():void {
      var copyString:String = "";
      for(var i:int=0; i<lines.length; i++) {
        copyString += lines[i].text + "\n";
      }
      System.setClipboard(copyString);
    }

// =============================================================================
// SETTINGS AND METRICS
// =============================================================================
    
    /**
     * <p>
     * Returns the number of lines that can currently be displayed in the text
     * field without scrolling.
     * </p>
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get pageLineCount():int {
      return Math.floor((textfield.height-borderThickness*2)/textfield.getLineMetrics(0).height);
    }
    
    /**
     * <p>
     * Returns the width of the text field, not including its border, in pixels.
     * </p>
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get pageWidth ():int {
      return textfield.width;
    }
    
    public function set scrollV (value:int):void {
      value = clamp(value, 1, maxScrollV);
      if (_scrollV != value) {
        _scrollV = value;
        dirtyText = true;
      }
    }
    
    /**
     * <p>
     * The line number currently displayed as the top line of the 
     * text field.
     * </p>
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get scrollV ():int {
      return _scrollV;
    }
    
    public function set scrollH (value:int):void {
      value = clamp(value, 0, maxScrollH);
      if (textfield.scrollH != value) {
        textfield.scrollH = value;
      }
    }
    
    /**
     * <p>
     * The amount, in pixels, the text in the text field is scrolled to the left.
     * </p>
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get scrollH ():int {
      return textfield.scrollH;
    }
    
    /**
     * <p>
     * The current maximum allowed value of scrollV.
     * </p>
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get maxScrollV ():int {
      return clamp(1 + lines.length - pageLineCount, 1, int.MAX_VALUE);
    }
    
    /**
     * <p>
     * The current maximum allowed value of scrollH.
     * </p>
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get maxScrollH ():int {
      return textfield.maxScrollH;
    }
    
    override public function set height (value:Number):void {
      dirtyCanvas = true;
      _height = value;
    }
    
    override public function get height ():Number {
      return _height;
    }
    
    override public function set width (value:Number):void {
      dirtyCanvas = true;
      _width = value;
    }
    
    override public function get width ():Number {
      return _width;
    }
    
    public function set backgroundColor (value:uint):void {
      dirtyCanvas = true;
      _backgroundColor = value;
    }
    
    public function get backgroundColor ():uint {
      return _backgroundColor;
    }

    public function get backgroundAlpha():Number {
      return _backgroundAlpha;
    }

    public function set backgroundAlpha(value:Number):void {
      dirtyCanvas = true;
      _backgroundAlpha = value;
    }

    public function get borderThickness():int {
      return _borderThickness;
    }

    public function set borderThickness(value:int):void {
      dirtyCanvas = true;
      _borderThickness = value;
    }

    public function get borderAlpha():Number {
      return _borderAlpha;
    }

    public function set borderAlpha(value:Number):void {
      dirtyCanvas = true;
      _borderAlpha = value;
    }

    public function get borderColor():uint {
      return _borderColor;
    }

    public function set borderColor(value:uint):void {
      dirtyCanvas = true;
      _borderColor = value;
    }

    /**
     * <p>
     * The current maximum number of lines allowed in the text field. When
     * this value is exceeded, adding a new last line of text automatically
     * causes the removal of the text field's current first line of text.
     * </p>
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get maxLength ():int {
      return _maxLength;
    }

    public function set maxLength (value:int):void {
      _maxLength = value;
    }

    /**
     * <p>
     * When true, the text field automatically stays scrolled to the last line.
     * </p>
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get autoScroll():Boolean {
      return _autoScroll;
    }

    public function set autoScroll(value:Boolean):void {
      _autoScroll = value;
    }

// =============================================================================
// HELPERS
// =============================================================================
    
    protected function clamp (value:Number, min:Number, max:Number):Number {
      if (value < min) return min;
      if (value > max) return max;
      return value;
    }

// =============================================================================
// DISPOSAL
// =============================================================================
    
    protected function dispose ():void {
      removeEventListener(Event.ENTER_FRAME, enterFrameListener);
      removeEventListener(MouseEvent.MOUSE_WHEEL, mouseWheelListener);
    }
  }
}