package net.user1.reactor.ui {
  import flash.events.Event;
  import flash.events.MouseEvent;
  
  import mx.containers.Form;
  import mx.containers.FormItem;
  import mx.containers.VBox;
  import mx.controls.Button;
  import mx.controls.ButtonLabelPlacement;
  import mx.controls.CheckBox;
  import mx.controls.Text;
  import mx.controls.TextInput;
  import mx.events.FlexEvent;
  
  import net.user1.prefs.Preferences;
  import net.user1.reactor.AccountEvent;
  import net.user1.reactor.AccountManager;
  import net.user1.reactor.Status;
  
  /**
   * Displays a user interface for logging in a user account.
   * Includes "User ID", "Password", and "Confirm Password" text fields,
   * plus a login button and "Remember password" checkbox.
   * 
   * @since ReactorGUI 1.0.0
   */
  public class LoginPane extends VBox {
//==============================================================================
// VARIABLES
//==============================================================================
    protected var _accountManager:AccountManager;
    
    protected var loginForm:Form;
    protected var useridInput:TextInput;
    protected var passwordInput:TextInput;
    protected var rememberPassCheck:CheckBox;
    protected var statusText:Text;
    
    protected var loginButton:Button;
    
    protected var prefs:Preferences;

//==============================================================================
// CONSTRUCTOR
//==============================================================================
    public function LoginPane () {
      prefs = new Preferences(LoginWindow);
      buildUI();
    }

//==============================================================================
// DEPENDENCIES
//==============================================================================
    /**
     * @private
     */
    public function set accountManager (value:AccountManager):void {
      if (_accountManager != null) {
        _accountManager.removeEventListener(AccountEvent.LOGIN_RESULT, loginResultListener);
      }
      _accountManager = value;
      
      if (_accountManager != null) {
        _accountManager.addEventListener(AccountEvent.LOGIN_RESULT, loginResultListener);
      }
      refresh();
    }
    
    /**
     * The AccountManager object used to perform login attempts.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get accountManager ():AccountManager {
      return _accountManager;
    }
    
//==============================================================================
// UI CREATION
//==============================================================================
    
    protected function buildUI ():void {
      var savedUserid:String = prefs.getPref("userid") != null
        ? String(prefs.getPref("userid"))
        : "";
      
      var savedPassword:String = prefs.getPref("password") != null
        ? String(prefs.getPref("password"))
        : "";
      
      loginForm = new Form();
      
      var useridItem:FormItem = new FormItem();
      useridItem.label = "User ID:";
      useridInput = new TextInput();
      useridInput.text = savedUserid != null ? savedUserid : "";
      useridInput.addEventListener(FlexEvent.ENTER, enterKeyListener);
      useridInput.addEventListener(Event.CHANGE, changeListener);
      useridItem.addChild(useridInput);
      
      var passwordItem:FormItem = new FormItem();
      passwordItem.label = "Password:";
      passwordInput = new TextInput();
      passwordInput.text = savedPassword;
      passwordInput.displayAsPassword = true;
      passwordInput.addEventListener(FlexEvent.ENTER, enterKeyListener);
      passwordInput.addEventListener(Event.CHANGE, changeListener);
      passwordItem.addChild(passwordInput);
      
      rememberPassCheck = new CheckBox();
      rememberPassCheck.label = "Remember password";
      rememberPassCheck.labelPlacement = ButtonLabelPlacement.LEFT;
      rememberPassCheck.setStyle("paddingRight", 0);
      rememberPassCheck.selected = prefs.getPref("password") != null;
      
      statusText = new Text();
      statusText.setStyle("textAlign", "center");
      statusText.setStyle("color", "0xAA0000");
      statusText.percentWidth = 100;
      
      loginButton = new Button();
      loginButton.label = "Login";
      loginButton.addEventListener(MouseEvent.CLICK, loginClickListener);
      loginButton.enabled = validate();
      
      
      loginForm.addChild(useridItem);
      loginForm.addChild(passwordItem);
      loginForm.addChild(rememberPassCheck);
      loginForm.addChild(loginButton);
      loginForm.addChild(statusText);
      loginForm.setStyle("horizontalAlign", "right");
      
      addChild(loginForm);
    }    

//==============================================================================
// UI CONTROL
//==============================================================================

    /**
     * Assigns input focus to the "User ID" field.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function focusUserID ():void {
      useridInput.setFocus();
    }

    protected function refresh ():void {
      if (_accountManager.selfAccount() != null
          && _accountManager.selfAccount().isLoggedIn()) {
        useridInput.enabled = false;
        passwordInput.enabled = false;
        rememberPassCheck.enabled = false;
        loginButton.enabled = false;
        statusText.text = "Already logged in.";
      } else {
        useridInput.enabled = true;
        passwordInput.enabled = true;
        rememberPassCheck.enabled = true;
        loginButton.enabled = true;
        statusText.text = "";
      }
    }
    
//==============================================================================
// FORM MANAGEMENT
//==============================================================================
    
    protected function validate ():Boolean {
      if (useridInput.length > 0 
        && passwordInput.length > 0) {
        return true;
      } else {
        return false;
      }
    }
    
    protected function submit ():void {
      statusText.text = "Logging in...";
      accountManager.login(useridInput.text, passwordInput.text);
      loginButton.enabled = false;
      savePrefs();
    }
    
    protected function savePrefs ():void {
      prefs.putPref("userid", useridInput.text);
      if (rememberPassCheck.selected) {
        prefs.putPref("password", passwordInput.text);
      } else {
        prefs.putPref("password", null);
      }
    }
    
//==============================================================================
// UI LISTENERS
//==============================================================================
    
    protected function loginClickListener (e:MouseEvent):void {
      submit();
    }
    
    protected function changeListener (e:Event):void {
      if (validate()) {
        loginButton.enabled = true;
      } else {
        loginButton.enabled = false;
      }
    }
    
    protected function enterKeyListener (e:FlexEvent):void {
      if (validate()) {
        submit();
      }
    }
    
//==============================================================================
// AccountEvent LISTENERS
//==============================================================================
    
    protected function loginResultListener (e:AccountEvent):void {
      loginButton.enabled = false;
      switch (e.getStatus()) {
        case Status.SUCCESS:
          statusText.text = "Logged in.";
          loginButton.enabled = true;
          break;
        case Status.ALREADY_LOGGED_IN:
          statusText.text = "Already logged in.";
          break;
        case Status.ACCOUNT_NOT_FOUND:
          statusText.text = "Account not found.";
          break;
        case Status.AUTHORIZATION_FAILED:
          statusText.text = "Authorization failed.";
          break;
        case Status.ERROR:
          statusText.text = "Login failed.";
          break;
      }
    }
    
//==============================================================================
// DISPOSAL
//==============================================================================
    
    /**
     * Disposes of the resources used by this object, preparing it for garbage
     * collection.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function dispose ():void {
      _accountManager.removeEventListener(AccountEvent.LOGIN_RESULT, loginResultListener);
    }
  }
}