package net.user1.reactor.ui {
  import flash.events.Event;
  import flash.events.MouseEvent;
  
  import mx.containers.Form;
  import mx.containers.FormItem;
  import mx.containers.VBox;
  import mx.controls.Button;
  import mx.controls.Text;
  import mx.controls.TextInput;
  import mx.events.FlexEvent;
  
  import net.user1.reactor.AccountManager;
  import net.user1.reactor.AccountManagerEvent;
  import net.user1.reactor.Status;
  
  /**
   * Displays a user interface for creating a user account.
   * Includes "User ID", "Password", and "Confirm Password" text fields, 
   * plus a "Create Account" button and a status field.
   * 
   * @since ReactorGUI 1.0.0
   * 
   * @see LoginPane
   */
  public class RegisterPane extends VBox {
    protected var _accountManager:AccountManager;
    
    protected var registerForm:Form;
    protected var useridInput:TextInput;
    protected var passwordInput1:TextInput;
    protected var passwordInput2:TextInput;
    protected var statusText:Text;
    
    protected var createAccountButton:Button;
    
    protected var invalidFormMessage:String;
    
    public function RegisterPane () {
      buildUI();
    }
    
//==============================================================================
// DEPENDENCIES
//==============================================================================
    /**
     * @private
     */    
    public function set accountManager (value:AccountManager):void {
      if (_accountManager != null) {
        _accountManager.removeEventListener(AccountManagerEvent.CREATE_ACCOUNT_RESULT, createAccountResultListener);
      }
      _accountManager = value;
      
      if (_accountManager != null) {
        _accountManager.addEventListener(AccountManagerEvent.CREATE_ACCOUNT_RESULT, createAccountResultListener);
      }
    }
    
    /**
     * The AccountManager object used to perform registration attempts.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get accountManager ():AccountManager {
      return _accountManager;
    }
    
//==============================================================================
// UI CREATION
//==============================================================================
    
    protected function buildUI ():void {
      registerForm = new Form();
      
      var useridItem:FormItem = new FormItem();
      useridItem.label = "User ID:";
      useridInput = new TextInput();
      useridInput.addEventListener(FlexEvent.ENTER, enterKeyListener);
      useridInput.addEventListener(Event.CHANGE, changeListener);
      useridItem.addChild(useridInput);
      
      var passwordItem1:FormItem = new FormItem();
      passwordItem1.label = "Choose a password:";
      passwordInput1 = new TextInput();
      passwordInput1.displayAsPassword = true;
      passwordInput1.addEventListener(FlexEvent.ENTER, enterKeyListener);
      passwordInput1.addEventListener(Event.CHANGE, changeListener);
      passwordItem1.addChild(passwordInput1);
      
      var passwordItem2:FormItem = new FormItem();
      passwordItem2.label = "Re-enter password:";
      passwordInput2 = new TextInput();
      passwordInput2.displayAsPassword = true;
      passwordInput2.addEventListener(FlexEvent.ENTER, enterKeyListener);
      passwordInput2.addEventListener(Event.CHANGE, changeListener);
      passwordItem2.addChild(passwordInput2);
      
      statusText = new Text();
      statusText.setStyle("textAlign", "center");
      statusText.setStyle("color", "0xAA0000");
      statusText.percentWidth = 100;
      
      createAccountButton = new Button();
      createAccountButton.label = "Create Account";
      createAccountButton.addEventListener(MouseEvent.CLICK, createAccountClickListener);
      createAccountButton.enabled = isFormComplete();
      
      registerForm.addChild(useridItem);
      registerForm.addChild(passwordItem1);
      registerForm.addChild(passwordItem2);
      registerForm.addChild(createAccountButton);
      registerForm.addChild(statusText);
      registerForm.setStyle("horizontalAlign", "right");
      
      addChild(registerForm);
    }    
    
//==============================================================================
// UI CONTROL
//==============================================================================
    
    /**
     * Assigns input focus to the "User ID" field.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function focusUserID ():void {
      useridInput.setFocus();
    }
    
//==============================================================================
// FORM MANAGEMENT
//==============================================================================
    
    protected function isFormComplete ():Boolean {
      if (useridInput.length > 0 
          && passwordInput1.length > 0
          && passwordInput2.length > 0) {
        return true;
      } else {
        return false;
      }
    }
    
    protected function validateForm ():Boolean {
      if (passwordInput1.text != passwordInput2.text) {
        invalidFormMessage = "Passwords do not match. Please try again.";
        return false;
      } else if (passwordInput1.text.indexOf(" ") != -1) {
        invalidFormMessage = "Password must not contain spaces. Please try again.";
        return false;
      } else if (useridInput.text.indexOf(" ") != -1) {
        invalidFormMessage = "User ID must not contain spaces. Please try again.";
        return false;
      } else if (useridInput.text == "") {
        invalidFormMessage = "User ID must not be empty. Please try again.";
        return false;
      }
      return true;
    }
    
    protected function submit ():void {
      if (passwordInput1.text != passwordInput2.text) {
      } else {
        statusText.text = "Creating account...";
        accountManager.createAccount(useridInput.text, passwordInput1.text);
        createAccountButton.enabled = false;
      }
    }
    
    protected function attemptSubmit ():void {
      if (validateForm()) {
        submit();
      } else {
        statusText.text = invalidFormMessage;
      }
    }
    
    protected function clearForm ():void {
      useridInput.text = "";
      passwordInput1.text = "";
      passwordInput2.text = "";
      useridInput.setFocus();
    }
    
//==============================================================================
// UI LISTENERS
//==============================================================================
    
    /**
     * Triggered when the form submit button is clicked.
     */    
    protected function createAccountClickListener (e:MouseEvent):void {
      attemptSubmit();
    }
    
    /**
     * Triggered when the text in the user ID or password fields changes.
     */
    protected function changeListener (e:Event):void {
      if (isFormComplete()) {
        createAccountButton.enabled = true;
      } else {
        createAccountButton.enabled = false;
      }
    }
    
    /**
     * Triggered when the enter key is pressed while any of the form fields
     * has focus.
     */
    protected function enterKeyListener (e:FlexEvent):void {
      attemptSubmit();
    }
    
//==============================================================================
// AccountManagerEvent LISTENERS
//==============================================================================
    
    /**
     * Triggered when Union server reports the results of an account-creation
     * request by the current client.
     */
    protected function createAccountResultListener (e:AccountManagerEvent):void {
      createAccountButton.enabled = true;
      switch (e.getStatus()) {
        case Status.SUCCESS:
          statusText.text = "Account created for user id: " + e.getUserID();
          clearForm();
          break;
        case Status.ACCOUNT_EXISTS:
          statusText.text = "Could not create account. User ID '" 
                          + e.getUserID() + "' is not available.";
          break;
        case Status.ERROR:
          statusText.text = "Could not create account. Server error.";
          break;
      }
    }
    
//==============================================================================
// DISPOSAL
//==============================================================================
    
    /**
     * Disposes of the resources used by this object, preparing it for garbage
     * collection.
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function dispose ():void {
      _accountManager.removeEventListener(AccountManagerEvent.CREATE_ACCOUNT_RESULT, createAccountResultListener);
    }
  }
}