package net.user1.reactor.ui {
  import mx.collections.ArrayCollection;
  import mx.collections.Sort;
  import mx.collections.SortField;
  import mx.controls.DataGrid;
  import mx.controls.dataGridClasses.DataGridColumn;
  
  import net.user1.reactor.AttributeEvent;
  import net.user1.reactor.Room;
  import net.user1.reactor.RoomEvent;

  /**
   * A data grid that displays a room's attributes. Used in
   * administrative and debugging interfaces.
   * 
   * @since 1.0.0
   *
   * @see ClientAttributeGrid
   */
  public class RoomAttributeGrid extends DataGrid {
// =============================================================================
// VARIABLES
// =============================================================================
    private var _room:Room;
    
// =============================================================================
// CONSTRUCTOR
// =============================================================================
    public function RoomAttributeGrid () {
      dataProvider = new ArrayCollection();
      var nameColumn:DataGridColumn = new DataGridColumn("Name");
      nameColumn.dataField = "name";
      nameColumn.width = .4;
      var valueColumn:DataGridColumn = new DataGridColumn("Value");
      valueColumn.dataField = "value";
      valueColumn.width = .6;
      columns = [nameColumn, valueColumn];

      var sort:Sort = new Sort();
      sort.fields = [new SortField("name",true)];
      dataProvider.sort = sort;
    }
    
// =============================================================================
// DEPENDENCIES
// =============================================================================
    
    /**
     * @private
     */    
    public function set room (newRoom:Room):void {
      if (_room != null) {
        _room.removeEventListener(RoomEvent.SYNCHRONIZE, synchronizeRoomListener);
        _room.removeEventListener(AttributeEvent.UPDATE, attributeUpdateListener);
        _room.removeEventListener(AttributeEvent.DELETE, attributeDeleteListener);
      }
      
      _room = newRoom;
      
      if (newRoom != null) {
        _room.addEventListener(RoomEvent.SYNCHRONIZE, synchronizeRoomListener);
        _room.addEventListener(AttributeEvent.UPDATE, attributeUpdateListener);
        _room.addEventListener(AttributeEvent.DELETE, attributeDeleteListener);
      }
      
      clearList();
      refreshDisplay();
    }
    
    /**
     * The Room whose attributes will be displayed by this RoomAttributeGrid.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get room ():Room {
      return _room;
    }  

// =============================================================================
// ROOM EVENTS
// =============================================================================
    
    /**
     * Triggered when the target Room's state is sychronized with the server.
     */
    protected function synchronizeRoomListener (e:RoomEvent):void {
      refreshDisplay();
    }
    
    /**
     * Triggered when the target Room sets an attribute.
     */
    protected function attributeUpdateListener (e:AttributeEvent):void {
      var attrName:String = e.getChangedAttr().name;
      var attrValue:String = e.getChangedAttr().value;
      var attrIndex:int = indexOfAttribute(attrName);

      if (attrIndex == -1) {
        addAttributeToGrid(attrName, attrValue);
      } else {
        dataProvider.getItemAt(attrIndex).value = attrValue;
        dataProvider.itemUpdated(dataProvider.getItemAt(attrIndex));
      }
    }
    
    /**
     * Triggered when the target Room deletes an attribute.
     */
    protected function attributeDeleteListener (e:AttributeEvent):void {
      dataProvider.removeItemAt(indexOfAttribute(e.getChangedAttr().name));
    }
    
// =============================================================================
// UI MANAGEMENT
// =============================================================================
    
    /**
     * Updates the data grid to reflect all attributes defined by the target
     * Room.
     */
    protected function refreshDisplay ():void {
      var attrs:Object;
      var attrIndex:int;
      
      if (_room != null) {
        attrs = _room.getAttributes();
        
        // Add all new attrs to the provider
        for (var attrName:String in attrs) {
          attrIndex = indexOfAttribute(attrName);
          if (attrIndex == -1) {
            addAttributeToGrid(attrName, attrs[attrName]);
          } else {
            dataProvider.getItemAt(attrIndex).value = attrs[attrName];
            dataProvider.itemUpdated(dataProvider.getItemAt(attrIndex));
          }
        }
        
        // Remove all missing attrs from the provider
        for (var i:int = dataProvider.length; --i >= 0;) {
          if (_room.getAttribute(dataProvider.getItemAt(i).name) == null) {
            dataProvider.removeItemAt(i);
          }
        }
      }
    }

//==============================================================================
// DATAPROVIDER HELPERS
//==============================================================================
    
    /**
     * Adds a new attribute to the grid.
     */    
    protected function addAttributeToGrid (attrName:String, attrValue:String):void {
      dataProvider.addItem({name:attrName, value:attrValue});
      dataProvider.refresh();
    }
    
    /**
     * Removes all attributes from the grid.
     */
    protected function clearList ():void {
      dataProvider.removeAll();
    }
    
    protected function indexOfAttribute (attrName:String):int {
      for (var i:int = dataProvider.length; --i >= 0;) {
        if (dataProvider.getItemAt(i).name == attrName) {
          return i;
        }
      }
      return -1;
    }
    
//==============================================================================
// DISPOSAL
//==============================================================================
    
    /**
     * Disposes of the resources used by this class, preparing it for garbage
     * collection.
     */    
    public function dispose ():void {
      if (_room != null) {
        _room.removeEventListener(RoomEvent.SYNCHRONIZE, synchronizeRoomListener);
        _room.removeEventListener(AttributeEvent.UPDATE, attributeUpdateListener);
        _room.removeEventListener(AttributeEvent.DELETE, attributeDeleteListener);
      }
      _room = null;
      dataProvider = null;
    }  
  }
}