package net.user1.reactor.ui {
  import mx.collections.ArrayCollection;
  import mx.collections.Sort;
  import mx.collections.SortField;
  import mx.controls.List;
  
  import net.user1.reactor.AccountEvent;
  import net.user1.reactor.AccountManager;
  import net.user1.reactor.IClient;
  import net.user1.reactor.Room;
  import net.user1.reactor.RoomEvent;
  import net.user1.reactor.SynchronizationState;
  import net.user1.reactor.UserAccount;
  import net.user1.utils.EventUtil;
  
  /**
   * The abstract base class for RoomOccupantList and RoomObserverList.
   * 
   * @since ReactorGUI 1.0.0
   */
  public class RoomClientList extends List {
// =============================================================================    
//  VARIABLES
// =============================================================================    
    protected var _room:Room;
    protected var _accountManager:AccountManager;
    
// =============================================================================    
//  CONSTRUCTOR    
// =============================================================================    
    public function RoomClientList () {
      dataProvider = new ArrayCollection();
      var sort:Sort = new Sort();
      sort.fields = [new SortField("label",true)];
      dataProvider.sort = sort;
    }
    
// =============================================================================    
//  DEPENDENCIES    
// =============================================================================    
    
    /**
     * @private
     */
    public function set accountManager (value:AccountManager):void {
      EventUtil.migrateListeners(_accountManager, value,
                                 [AccountEvent.LOGIN, loginListener,
                                  AccountEvent.LOGOFF, logoffListener]);
      _accountManager = value;
    }
    
    /**
     * The AccountManager that this UserList uses to determine user IDs for 
     * clients in the list. 
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get accountManager ():AccountManager {
      return _accountManager;
    }
    
    /**
     * @private
     */
    public function set room (value:Room):void {
      migrateListeners(_room, value);
      _room = value;
      clearList();
      refreshDisplay();
    }
    
    /**
     * The Room whose clients will be displayed in this UserList. 
     * 
     * @since ReactorGUI 1.0.0
     */
    public function get room ():Room {
      return _room;
    }
    
    protected function migrateListeners (oldRoom:Room, newRoom:Room):void {
      EventUtil.migrateListeners(oldRoom, newRoom,
        [RoomEvent.SYNCHRONIZE, synchronizeRoomListener]);
    }
    
    protected function disposeListeners ():void {
      _room.removeEventListener(RoomEvent.SYNCHRONIZE, synchronizeRoomListener);
    }
    
// =============================================================================
// ACCOUNTMANAGER EVENT LISTENERS
// =============================================================================
    
    /**
     * Triggered when a client logs in.
     */
    protected function loginListener (e:AccountEvent):void {
      if (_room == null) {
        return;
      }
      
      // Change the username in the user list.
      var updatedClient:IClient = _room.getClient(e.getClientID());
      if (updatedClient != null) {
        for (var i:int = 0; i < dataProvider.length; i++) {
          if (dataProvider.getItemAt(i).data == updatedClient) {
            // Found the client, so change the name.
            dataProvider.getItemAt(i).label = getUserName(updatedClient);
            // The name changed, so resort.
            dataProvider.refresh();
            break;
          } 
        }
      }
    }
    
    /**
     * Triggered when a client logs off.
     */
    protected function logoffListener (e:AccountEvent):void {
    }
    
// =============================================================================
// ROOM EVENT LISTENERS
// =============================================================================
    
    /**
     * Triggered when a client joins the room.
     */
    protected function addClientListener (e:RoomEvent):void {
      // Ignore updates during a room synchronization. We'll synchronize the
      // UI's dataprovider later in the synchronizeRoomListener() method.
      if (room.getSyncState() == SynchronizationState.SYNCHRONIZING) {
        return;
      }
      
      var client:IClient = e.getClient();
      
      // Don't re-add any client that is in the dataprovider already
      for (var i:int = 0; i < dataProvider.length; i++) {
        if (IClient(dataProvider.getItemAt(i).data).getClientID() 
            == client.getClientID()) {
          return;
        }
      }

      addClientToList(client);
    }
    
    /**
     * Triggered when a client leaves the room.
     */
    protected function removeClientListener (e:RoomEvent):void {
      removeClientFromList(e.getClient());
    }
    
    /**
     * Triggered when the local room has been synchronized with the server.
     */
    protected function synchronizeRoomListener (e:RoomEvent):void {
      refreshDisplay();
    }
    
// =============================================================================
// ROOM DATA ACCESS
// =============================================================================
    
    protected function getClients ():Array {
      return [];
    }
    
// =============================================================================
// UI MANAGEMENT
// =============================================================================
  
    /**
     * Synchronizes this UserList with the target Room's client list.
     */    
    protected function refreshDisplay ():void {
      var clientExists:Boolean;
      var clients:Array;
      
      if (_room != null) {
        clients = getClients();
        // Add all new clients to the provider
        for each (var client:IClient in clients) {
          clientExists = false;
          for (var i:int = dataProvider.length; --i >= 0;) {
            if (dataProvider.getItemAt(i).data == client) {
              clientExists = true;
              break;
            }
          }
          if (!clientExists) {
            addClientToList(client);
          }
        }
        
        // Remove all missing clients from the provider
        for (i = dataProvider.length; --i >= 0;) {
          clientExists = false;
          for each (client in clients) {
            if (client == dataProvider.getItemAt(i).data) {
              clientExists = true;
              break;
            }
          }
          if (!clientExists) {
            dataProvider.removeItemAt(i);
          }
        }
      }
    }

    /**
     * Removes all content from the user list.
     */
    protected function clearList ():void {
      dataProvider.removeAll();
    }

    /**
     * Adds a user to the on-screen user list for the room.
     */
    protected function addClientToList (client:IClient):void {
      // Insert the client into the user list.
      dataProvider.addItem({label:getUserName(client), data:client});
      dataProvider.refresh();
    }

    /**
     * Removes a user from the on-screen user list for the room.
     */
    protected function removeClientFromList (client:IClient):void {
      // Remove the client from the user list.
      for (var i:int = 0; i < dataProvider.length; i++) {
        if (IClient(dataProvider.getItemAt(i).data).getClientID() 
            == client.getClientID()) {
            dataProvider.removeItemAt(i);
            dataProvider.refresh();
            return;
        }
      }
      throw new Error("[" + this 
                      + "] Remove client requested but client is not in the list: " 
                      + client);
    }
    
    /**
     * Returns the client's user ID, or if no user ID has been set, returns the
     * string: "User" + the client's ID (e.g., "User24").
     */ 
    protected function getUserName (client:IClient):String {
      var username:String = "";
      var userAccount:UserAccount = client.getAccount();
      if (userAccount == null) {
        if (client.isSelf()) {
          username = "User" + client.getClientID() + " [You]";
        } else {
          username = "User" + client.getClientID();
        }
      } else {
        if (client.isSelf()) {
          username = client.getAccount().getUserID() + " [You]";
        } else {
          username = client.getAccount().getUserID();
        }
      }
      return username;
    }    
    
// =============================================================================
// DISPOSAL
// =============================================================================

    /**
     * Disposes of the resources used by this class, preparing it for garbage
     * collection.
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function dispose ():void {
      if (_room != null) {
        disposeListeners();
      }
      _room = null;
      dataProvider = null;
    }
  }
}
