package net.user1.reactor.ui {
  import mx.collections.ArrayCollection;
  import mx.collections.Sort;
  import mx.collections.SortField;
  import mx.controls.List;
  
  import net.user1.reactor.Room;
  import net.user1.reactor.RoomManager;
  import net.user1.reactor.RoomManagerEvent;
  
  /**
   * A List component that displays all rooms in the RoomManager's room list, 
   * or just those rooms that share a specified qualifier.
   * 
   * @since ReactorGUI 1.0.0
   */
  public class RoomList extends List {
// =============================================================================
// VARIABLES
// =============================================================================
    private var _roomManager:RoomManager;
    private var _qualifier:String;
    
// =============================================================================
// CONSTRUCTOR
// =============================================================================
    /**
     * Constructor
     */
    public function RoomList () {
      dataProvider = new ArrayCollection();
      var sort:Sort = new Sort();
      sort.fields = [new SortField("label",true)];
      dataProvider.sort = sort;
    }
    
// =============================================================================
// DEPENDENCIES
// =============================================================================
    /**
     * A Reactor RoomManager reference that provides 
     * the underlying data for the visual room list.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function set roomManager (value:RoomManager):void {
      if (_roomManager != null) {
        _roomManager.removeEventListener(RoomManagerEvent.ROOM_ADDED, roomAddedListener);
        _roomManager.removeEventListener(RoomManagerEvent.ROOM_REMOVED, roomRemovedListener);
      }

      _roomManager = value;
      
      if (_roomManager != null) {
        _roomManager.addEventListener(RoomManagerEvent.ROOM_ADDED, roomAddedListener);
        _roomManager.addEventListener(RoomManagerEvent.ROOM_REMOVED, roomRemovedListener);
      }
      refreshDisplay();
    }
    
    /**
     * @private
     */
    public function get roomManager ():RoomManager {
      return _roomManager;
    }
    
    /**
     * The qualifier of the rooms that should be included in this room list.
     * To include all rooms on in the RoomManager's room list, 
     * set the qualifier to null.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function set qualifier (value:String):void {
      _qualifier = value;
      refreshDisplay();
    }
    
    /**
     * @private
     */
    public function get qualifier ():String {
      return _qualifier;
    }
	
// =============================================================================
// ROOMMANAGER EVENT LISTENERS
// =============================================================================
    
    /**
     * Triggered when a room is added to the room list.
     */
    protected function roomAddedListener (e:RoomManagerEvent):void {
      addRoomToList(e.getRoom());
    }
    
    /**
     * Triggered when a room is removed from the room list.
     */
    protected function roomRemovedListener (e:RoomManagerEvent):void {
      removeRoomFromList(e.getRoom().getRoomID());
    }
    
// =============================================================================
// UI MANAGEMENT
// =============================================================================
  
    protected function refreshDisplay ():void {
      clearList();
      if (_roomManager != null) {
        for each (var room:Room in _roomManager.getRoomsWithQualifier()) {
          addRoomToList(room);
        }
      }
    }

    /**
     * Removes all content from the room list.
     */
    protected function clearList ():void {
      dataProvider.removeAll();
    }

    /**
     * Adds a room to the on-screen room list.
     */
    protected function addRoomToList (room:Room):void {
      // Display the room only if its qualifier matches the qualifier this list
      // is interested in
      if (qualifier != null && room.getQualifier() != qualifier) {
        return;
      }
      // Insert the room into the list.
      dataProvider.addItem({label:room.getRoomID(), data:room});
      dataProvider.refresh();
    }

    /**
     * Removes a room from the on-screen room list.
     */
    protected function removeRoomFromList (roomID:String):void {
      // Remove the room from the user list.
      for (var i:int = 0; i < dataProvider.length; i++) {
        if (Room(dataProvider.getItemAt(i).data).getRoomID() 
            == roomID) {
            dataProvider.removeItemAt(i);
        }
      }

      dataProvider.refresh();
    }

// =============================================================================
// DISPOSAL
// =============================================================================
	
    /**
     * Disposes of the resources used by this object, preparing it for garbage
     * collection.
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function dispose ():void {
      if (_roomManager != null) {
        _roomManager.removeEventListener(RoomManagerEvent.ROOM_ADDED, roomAddedListener);
        _roomManager.removeEventListener(RoomManagerEvent.ROOM_REMOVED, roomRemovedListener);
      }
      _roomManager = null;
      dataProvider = null;
    }
  }
}
