package net.user1.reactor.ui {
  import flash.events.Event;
  import flash.events.TimerEvent;
  import flash.utils.Timer;
  import flash.utils.getTimer;
  
  import mx.containers.VBox;
  import mx.controls.TextArea;
  
  import net.user1.reactor.Client;
  import net.user1.reactor.Reactor;
  import net.user1.reactor.ReactorEvent;
  import net.user1.reactor.Statistics;
  
  /**
   * Provides an on-screen display of application and connection statistics for
   * a Reactor application, including message count, bandwidth usage, memory, 
   * and framerate.
   * 
   * @since ReactorGUI 1.0.0
   * 
   * @see DebugPanel
   */
  public class StatsPane extends VBox {
// =============================================================================
// VARIABLES
// =============================================================================
    private var reactor:Reactor;
    private var stats:Statistics;
    private var statsList:Array;
    private var statsOutput:TextArea;
    private var statsTimer:Timer;
    private var fps:Number;
    private var lastFrameTick:int;

// =============================================================================
// CONSTRUCTOR
// =============================================================================
    public function StatsPane () {
      statsList = new Array();
      statsTimer = new Timer(1000, 0);
      percentWidth = 100;
      percentHeight = 100;
    }
    
// =============================================================================
// UI MANAGEMENT
// =============================================================================
    override protected function createChildren ():void {
      super.createChildren();
      if (statsOutput == null) {
        statsOutput = new AutoScrollTextArea();
        statsOutput.percentWidth = 100;
        statsOutput.percentHeight = 100;
        
        addChild(statsOutput);
      }
    }
    
    protected function clearDefaultStats ():void {
      statsList = [];
    }
    

// =============================================================================
// STATISTIC REGISTRATION
// =============================================================================
    
    /**
     * Adds a new statistic to the StatsPane. Each statistic must have a
     * string name and a corresponding function that returns the value
     * for that statistic. The StatsPane periodically calculates and displays
     * the value of each statistic by calling the statistic's function. For
     * example, the following code adds a "Connection Attempts" statistic that
     * displays the number of times a Reactor application has attempted to 
     * connect to Union. The statistic's value is returned by the 
     * ConnectionManager class's built-in method, getConnectAttemptCount().
     * 
     * <listing>
     * addStat("Connection Attempts", reactor.getConnectionManager().getConnectAttemptCount); 
     * </listing>
     * 
     * @param name The name of the statistic to add to the StatsPane.
     * @param func The function that calculates and returns the statistic's 
     *             value.
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function addStat (name:String, func:Function):void {
      if (func != null) {
        statsList.push([name, func]);
      }
    }

    /**
     * Adds the default set of statistics. 
     */    
    protected function setDefaultStats ():void {
      addStat("Client ID", getClientID); 
      addStat("Connection Attempts", reactor.getConnectionManager().getConnectAttemptCount); 
      addStat("Connection Aborts", reactor.getConnectionManager().getConnectAbortCount); 
      addStat("Connection Failures", reactor.getConnectionManager().getConnectFailedCount); 
      addStat("Connection Successes", reactor.getConnectionManager().getReadyCount); 
      addStat("Current Clients", calcCurrentClients); 
      addStat("Lifetime Clients", calcLifetimeClients); 
      addStat("Memory", calcMemory); 
      addStat("Messages Sent", stats.getTotalMessagesSent);
      addStat("Messages received", stats.getTotalMessagesReceived);
      addStat("Messages/second", calcMessagesPerSecond);
      addStat("MB sent", calcMBSent);
      addStat("MB recd", calcMBReceived);
      addStat("KB sent/second", calcKBSentPerSecond);
      addStat("KB recd/second", calcKBReceivedPerSecond);
      addStat("KB total/second", calcKBTotalPerSecond);
      addStat("Ping", calcPing);
      addStat("FPS", calcFPS);
    }
    
// =============================================================================
// OUTPUT GENERATION
// =============================================================================
    
    private function getClientID ():String {
      return reactor.isReady() ? reactor.self().getClientID() : "-";
    }
    
    private function calcCurrentClients ():String {
      return reactor.isReady() ? stats.getCurrentNumClientsConnected().toString() : "-";
    }
    
    private function calcLifetimeClients ():String {
      return reactor.isReady() ? stats.getLifetimeNumClientsConnected().toString() : "-";
    }
    
    private function calcMemory ():String {
      return String(stats.getTotalMemoryMB()) + " MB" + " (Peak: " + stats.getPeakMemoryMB() + ")";
    }
    
    private function calcMessagesPerSecond ():String {
      return reactor.isReady() ? String(stats.getMessagesPerSecond())  + " (Peak: " + stats.getPeakMessagesPerSecond() + ")" : "-";
    }
    
    private function calcMBSent ():String {
      return reactor.isReady() ? stats.getTotalMBSent().toString() : "-"; 
    }
    
    private function calcMBReceived ():String {
      return reactor.isReady() ? stats.getTotalMBReceived().toString() : "-"; 
    }
    
    private function calcKBSentPerSecond ():String {
      return reactor.isReady() ? String(stats.getKBSentPerSecond()) + " (Peak: " + stats.getPeakKBSentPerSecond() + ")" : "-";
    }
    
    private function calcKBReceivedPerSecond ():String {
      return reactor.isReady() ? String(stats.getKBReceivedPerSecond()) + " (Peak: " + stats.getPeakKBReceivedPerSecond() + ")" : "-";
    }
    
    private function calcKBTotalPerSecond ():String {
      return reactor.isReady() ? String(stats.getKBPerSecond()) + " (Peak: " + stats.getPeakKBPerSecond() + ")" : "-";
    }
    
    private function calcFPS ():Number {
      return fps;
    }
    
    private function calcPing ():String {
      if (reactor.self() != null) {
        return reactor.self().getPing().toString();
      } else {
        return "-"
      }
    }

// =============================================================================
// DEPENDENCIES
// =============================================================================
    
    /**
     * Assigns the Reactor object for which this StatsPane will display 
     * statistics.
     * 
     * @since ReactorGUI 1.0.0
     */
    public function setReactor (reactor:Reactor):void {
      this.reactor = reactor;
      start();
    }
    
// =============================================================================
// ACTIVATION/DEACTIVATION
// =============================================================================
    
    /**
     * Activates the statistics pane. 
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function start ():void {
      stop();
      lastFrameTick = getTimer();
      stats = reactor.getStatistics();
      addEventListener(Event.ENTER_FRAME, enterFrameListener);
      
      // If statistics are disabled for the connection, enable them.
      if (stats == null) {
        reactor.enableStatistics();
        stats = reactor.getStatistics();
      }
      setDefaultStats();

      statsTimer.addEventListener(TimerEvent.TIMER, statsTimerListener);
      statsTimer.start();
    }
    
    /**
     * Deactivates the statistics pane. 
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function stop ():void {
      stats = null;
      if (statsTimer != null) {
        statsTimer.reset();
        statsTimer.removeEventListener(TimerEvent.TIMER, statsTimerListener);
      }
      removeEventListener(Event.ENTER_FRAME, enterFrameListener);
      clearDefaultStats();
    }

    private function statsTimerListener (e:TimerEvent):void {
      statsOutput.text = "";
      for each (var stat:Array in statsList) {
        statsOutput.text += stat[0] + ": " + String(stat[1]()) + "\n";
      } 
    }
    
    private function enterFrameListener (e:Event):void {
      var now:int = getTimer();
      var elapsed:int = now - lastFrameTick;
      lastFrameTick = now;
      fps = Math.floor(10000/elapsed)/10;
    }
    
// =============================================================================
// DISPOSAL
// =============================================================================
    
    /**
     * Disposes of the resources used by this object, preparing it for garbage
     * collection.
     * 
     * @since ReactorGUI 1.0.0
     */    
    public function dispose ():void {
      stop();
      reactor = null;
      statsTimer = null;
      statsList = null;
    }
  }
}
