package net.user1.reactor.ui {
  import flash.events.KeyboardEvent;
  import flash.events.MouseEvent;
  import flash.ui.Keyboard;
  
  import mx.containers.HBox;
  import mx.containers.TitleWindow;
  import mx.controls.Button;
  import mx.controls.TextInput;
  import mx.events.FlexEvent;
  import mx.managers.PopUpManager;
  
  import net.user1.reactor.Room;
  import net.user1.reactor.chat.ChatAttributes;
  
  /**
   * A general-purpose popup window for receiving text input in an application.
   * Includes a text field, an OK button, and a Cancel button. The name of the
   * input field should be specified in the window title. Clicking a
   * TextInputWindow's OK button triggers a TextInputWindowEvent.SUBMIT event.
   * 
   * @example
   * The following code creates a TextInputWindow to set a chat room topic:
   * 
   * <listing>
   * var popup:TextInputWindow = TextInputWindow(
   *                      PopUpManager.createPopUp(this, TextInputWindow, true));
   * popup.title = "Set New Topic";
   * popup.addEventListener(TextInputWindowEvent.SUBMIT, submitListener);      
   * PopUpManager.centerPopUp(popup);
   * </listing>
   * 
   * @since ReactorGUI 1.0.0
   */
  public class TextInputWindow extends TitleWindow {
// =============================================================================
// VARIABLES
// =============================================================================
    private var ok:Button;
    private var cancel:Button;
    private var input:TextInput;

// =============================================================================
// CONSTRUCTOR
// =============================================================================
    /**
     * Constructor
     */
    public function TextInputWindow () {
      buildUI();
    }
    
// =============================================================================
// UI MANAGEMENT
// =============================================================================
    protected function buildUI ():void {
      this.addEventListener(FlexEvent.CREATION_COMPLETE, creationCompleteListener);
      
      input = new TextInput();
      input.addEventListener(KeyboardEvent.KEY_DOWN, inputKeyListener);
      
      ok = new Button();
      ok.label = "OK";
      ok.addEventListener(MouseEvent.CLICK, okClickListener);
      
      cancel = new Button();
      cancel.label = "Cancel";
      cancel.addEventListener(MouseEvent.CLICK, cancelClickListener);
      
      var buttonBox:HBox = new HBox();
      buttonBox.setStyle("horizontalAlign", "right");
      buttonBox.percentWidth = 100;
      buttonBox.addChild(ok);
      buttonBox.addChild(cancel);
      
      addChild(input);
      addChild(buttonBox);
    }

    protected function creationCompleteListener (e:FlexEvent):void {
      input.setFocus();
    }
    
// =============================================================================
// UI LISTENERS
// =============================================================================
    protected function okClickListener (e:MouseEvent):void {
      submitData();
    }

    protected function cancelClickListener (e:MouseEvent):void {
      close();
    }

    /**
     * Listener triggered whenever a key is pressed while the "input" TextInput
     * component has focus. This listener lets the user use the ENTER key instead
     * of pressing OK.
     */ 
    protected function inputKeyListener (e:KeyboardEvent):void {
      if (e.keyCode == Keyboard.ENTER) {
        submitData();
      }
    }
    
// =============================================================================
// SUBMIT
// =============================================================================
    
    protected function submitData ():void {
      dispatchSubmit();
      close();
    }
    
// =============================================================================
// CLOSE
// =============================================================================
    
    /**
     * Closes this popup window by calling PopUpManager.removePopUp(this).
     * 
     * @since ReactorGUI 1.0.0
     */
    public function close ():void {
      PopUpManager.removePopUp(this);
    }
    
// =============================================================================
// EVENT DISPATCHING
// =============================================================================
    
    protected function dispatchSubmit ():void {
      dispatchEvent(new TextInputWindowEvent(TextInputWindowEvent.SUBMIT,
                                             false, false, input.text));
    }
  }
}